"""Testing formula 6.9 of EN 1993-1-1:2005."""

import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_6_ultimate_limit_state.formula_6_9 import Form6Dot9CheckCompressionForce
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm6Dot9CheckCompressionForce:
    """Validation for formula 6.9 from EN 1993-1-1:2005."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        n_ed = 100.0
        n_c_rd = 150.0

        # Object to test
        formula = Form6Dot9CheckCompressionForce(n_ed=n_ed, n_c_rd=n_c_rd)

        # Expected result, manually calculated
        expected_result = True

        assert formula == expected_result

    @pytest.mark.parametrize(
        ("n_ed", "n_c_rd"),
        [
            (100.0, 0.0),  # n_c_rd is zero
            (-100.0, 150.0),  # n_ed is negative
            (100.0, -150.0),  # n_c_rd is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, n_ed: float, n_c_rd: float) -> None:
        """Test invalid values."""
        with pytest.raises((LessOrEqualToZeroError, NegativeValueError)):
            Form6Dot9CheckCompressionForce(n_ed=n_ed, n_c_rd=n_c_rd)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"CHECK \to \left( \frac{N_{Ed}}{N_{c,Rd}} \leq 1 \right) \to "
                r"\left( \frac{100.000}{150.000} \leq 1 \right) \to OK",
            ),
            ("short", r"CHECK \to OK"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        n_ed = 100.0
        n_c_rd = 150.0

        # Object to test
        latex = Form6Dot9CheckCompressionForce(n_ed=n_ed, n_c_rd=n_c_rd).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Formula 3.1 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_negative


class Form3Dot1EstimationConcreteCompressiveStrength(Formula):
    r"""Class representing formula 3.1 for the estimation of the concrete compressive strength, [$f_{cm}(t)$], after t days
    with an average temperature of 20 degrees Celsius.
    """

    label = "3.1"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        beta_cc_t: DIMENSIONLESS,
        f_cm: MPA,
    ) -> None:
        r"""[$f_{cm}(t)$] The estimated concrete compressive strength [$MPa$].

        EN 1992-1-1:2004 art.3.1.2(6) - Formula (3.1)

        Parameters
        ----------
        beta_cc_t : DIMENSIONLESS
            [$\beta_{cc}(t)$] Coefficient dependent of the age of concrete [$-$].
        f_cm : MPA
            [$f_{cm}$] Average concrete compressive strength on day 28 based on table 3.1 [$MPa$].

        Returns
        -------
        None
        """
        super().__init__()
        self.beta_cc_t = beta_cc_t
        self.f_cm = f_cm

    @staticmethod
    def _evaluate(
        beta_cc_t: DIMENSIONLESS,
        f_cm: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(beta_cc_t=beta_cc_t, f_cm=f_cm)
        return beta_cc_t * f_cm

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 3.1."""
        return LatexFormula(
            return_symbol=r"f_{cm}(t)",
            result=f"{self:.3f}",
            equation=r"\beta_{cc}(t) \cdot f_{cm}",
            numeric_equation=rf"{self.beta_cc_t:.3f} \cdot {self.f_cm:.3f}",
            comparison_operator_label="=",
        )



"""Tests for the Exposure classes."""

import pytest

from blueprints.codes.eurocode.exposure_classes import (
    CarbonationBase,
    ChemicalBase,
    ChlorideBase,
    ChlorideSeawaterBase,
    Exposure,
    ExposureClassesBase,
    FreezeThawBase,
)


class DummyExposureSubclass(Exposure):
    """Dummy Exposure subclass for testing purposes."""

    DUMMY1 = "Dummy1"
    DUMMY2 = "Dummy2"
    DUMMY3 = "Dummy3"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyExposureSubclass."""
        return "Dummy exposure subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyCarbonation(CarbonationBase):
    """Dummy Carbonation subclass for testing purposes."""

    NA = "Not applicable"
    XC1 = "XC1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyCarbonation."""
        return "Dummy carbonation subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyChloride(ChlorideBase):
    """Dummy Chloride subclass for testing purposes."""

    NA = "Not applicable"
    XD1 = "XD1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChloride."""
        return "Dummy chloride subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyChlorideSeawater(ChlorideSeawaterBase):
    """Dummy ChlorideSeawater subclass for testing purposes."""

    NA = "Not applicable"
    XS1 = "XS1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChlorideSeawater."""
        return "Dummy chloride seawater subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyFreezeThaw(FreezeThawBase):
    """Dummy FreezeThaw subclass for testing purposes."""

    NA = "Not applicable"
    XF1 = "XF1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyFreezeThaw."""
        return "Dummy freeze thaw subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class DummyChemical(ChemicalBase):
    """Dummy Chemical subclass for testing purposes."""

    NA = "Not applicable"
    XA1 = "XA1"

    @staticmethod
    def exposure_class_description() -> str:
        """Return the description of the DummyChemical."""
        return "Dummy chemical subclass"

    def description_of_the_environment(self) -> str:
        """Return the description of the environment."""
        return "Dummy environment"


class TestExposure:
    """Testing Exposure parent class."""

    def test_equal_to_operator(self) -> None:
        """Check if the == operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 == DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY2 == DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY3 == DummyExposureSubclass.DUMMY3

    def test_not_equal_to_operator(self) -> None:
        """Check if the != operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 != DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 != DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.DUMMY3 != DummyExposureSubclass.DUMMY1

    def test_greather_than_operator(self) -> None:
        """Check if the > operator is working correctly."""
        assert DummyExposureSubclass.DUMMY3 > DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 > DummyExposureSubclass.DUMMY1

    def test_greather_than_equal_to_operator(self) -> None:
        """Check if the >= operator is working correctly."""
        # Testing greater than
        assert DummyExposureSubclass.DUMMY3 >= DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 >= DummyExposureSubclass.DUMMY1
        # Testing equal to
        assert DummyExposureSubclass.DUMMY3 >= DummyExposureSubclass.DUMMY3
        assert DummyExposureSubclass.DUMMY1 >= DummyExposureSubclass.DUMMY1

    def test_lesser_than_operator(self) -> None:
        """Check if the < operator is working correctly."""
        assert DummyExposureSubclass.DUMMY1 < DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 < DummyExposureSubclass.DUMMY3

    def test_lesser_than_equal_to_operator(self) -> None:
        """Check if the <= operator is working correctly."""
        # Testing lesser than
        assert DummyExposureSubclass.DUMMY1 <= DummyExposureSubclass.DUMMY2
        assert DummyExposureSubclass.DUMMY2 <= DummyExposureSubclass.DUMMY3
        # Testing equal to
        assert DummyExposureSubclass.DUMMY1 <= DummyExposureSubclass.DUMMY1
        assert DummyExposureSubclass.DUMMY3 <= DummyExposureSubclass.DUMMY3

    def test_options(self) -> None:
        """Check if the options method returns all the possible options within an exposure class."""
        assert DummyExposureSubclass.options() == ["Dummy1", "Dummy2", "Dummy3"]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyExposureSubclass.exposure_class_description() == "Dummy exposure subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyExposureSubclass.DUMMY1.description_of_the_environment() == "Dummy environment"


class TestCarbonation:
    """Testing CarbonationBase class."""

    def test_initiate_subclasses(self) -> None:
        """Check if initiating the CarbonationBase class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = CarbonationBase()  # type: ignore[abstract, call-arg]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyCarbonation.exposure_class_description() == "Dummy carbonation subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyCarbonation.XC1.description_of_the_environment() == "Dummy environment"


class TestChloride:
    """Testing ChlorideBase class."""

    def test_initiate_subclasses(self) -> None:
        """Check if initiating the ChlorideBase class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = ChlorideBase()  # type: ignore[abstract, call-arg]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChloride.exposure_class_description() == "Dummy chloride subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChloride.XD1.description_of_the_environment() == "Dummy environment"


class TestChlorideSeawater:
    """Testing ChlorideSeawaterBase class."""

    def test_initiate_subclasses(self) -> None:
        """Check if initiating the ChlorideSeawaterBase class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = ChlorideSeawaterBase()  # type: ignore[abstract, call-arg]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChlorideSeawater.exposure_class_description() == "Dummy chloride seawater subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChlorideSeawater.XS1.description_of_the_environment() == "Dummy environment"


class TestFreezeThaw:
    """Testing FreezeThawBase class."""

    def test_initiate_subclasses(self) -> None:
        """Check if initiating the FreezeThawBase class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = FreezeThawBase()  # type: ignore[abstract, call-arg]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyFreezeThaw.exposure_class_description() == "Dummy freeze thaw subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyFreezeThaw.XF1.description_of_the_environment() == "Dummy environment"


class TestChemical:
    """Testing ChemicalBase class."""

    def test_initiate_subclasses(self) -> None:
        """Check if initiating the ChemicalBase class raises a TypeError."""
        with pytest.raises(TypeError):
            _ = ChemicalBase()  # type: ignore[abstract, call-arg]

    def test_exposure_class_description_implemented(self) -> None:
        """Check if the exposure_class_description method returns the description of the subclass."""
        assert DummyChemical.exposure_class_description() == "Dummy chemical subclass"

    def test_description_of_the_environment(self) -> None:
        """Check if the description_of_the_environment method returns the description of the environment."""
        assert DummyChemical.XA1.description_of_the_environment() == "Dummy environment"


class TestExposureClasses:
    """Testing ExposureClasses class."""

    def test_str(self) -> None:
        """Check if the __str__ method returns the correct string representation of the exposure classes."""
        exposure_classes = ExposureClassesBase(
            carbonation=DummyCarbonation("XC1"),
            chloride=DummyChloride("XD1"),
            chloride_seawater=DummyChlorideSeawater("Not applicable"),
            freeze=DummyFreezeThaw("XF1"),
            chemical=DummyChemical("XA1"),
        )
        assert str(exposure_classes) == "XC1, XD1, XF1, XA1"

    def test_str_x0(self) -> None:
        """Check if the __str__ method returns the correct string representation of the exposure classes
        when all exposure classes are not applicable.
        """
        exposureclasses = ExposureClassesBase(
            carbonation=DummyCarbonation("Not applicable"),
            chloride=DummyChloride("Not applicable"),
            chloride_seawater=DummyChlorideSeawater("Not applicable"),
            freeze=DummyFreezeThaw("Not applicable"),
            chemical=DummyChemical("Not applicable"),
        )
        assert str(exposureclasses) == "X0"

    def test_no_risk(self) -> None:
        """Check if the no_risk method returns True if the exposure classes are all not applicable."""
        exposureclasses = ExposureClassesBase(
            carbonation=DummyCarbonation("Not applicable"),
            chloride=DummyChloride("Not applicable"),
            chloride_seawater=DummyChlorideSeawater("Not applicable"),
            freeze=DummyFreezeThaw("Not applicable"),
            chemical=DummyChemical("Not applicable"),
        )
        assert exposureclasses.no_risk is True

    def test_no_risk_false(self) -> None:
        """Check if the no_risk method returns False if at least one exposure class is applicable."""
        exposureclasses = ExposureClassesBase(
            carbonation=DummyCarbonation("XC1"),
            chloride=DummyChloride("XD1"),
            chloride_seawater=DummyChlorideSeawater("Not applicable"),
            freeze=DummyFreezeThaw("XF1"),
            chemical=DummyChemical("XA1"),
        )
        assert exposureclasses.no_risk is False


def test_comparing_different_types_raises_error() -> None:
    """Check if comparing different exposure class types, raises TypeError."""
    with pytest.raises(TypeError):
        DummyCarbonation.XC1 > DummyChloride.NA
    with pytest.raises(TypeError):
        DummyChloride.NA == DummyChemical.NA
    with pytest.raises(TypeError):
        DummyFreezeThaw.XF1 <= DummyChlorideSeawater.XS1
    with pytest.raises(TypeError):
        DummyChlorideSeawater.XS1 >= DummyChloride.XD1
    with pytest.raises(TypeError):
        DummyChemical.XA1 < DummyCarbonation.XC1
    with pytest.raises(TypeError):
        DummyChemical.NA != DummyFreezeThaw.XF1



"""Testing formula 5.48 of EN 1992-1-1:2004."""

import pytest

from blueprints.codes.eurocode.en_1992_1_1_2004.chapter_5_structural_analysis.formula_5_48 import (
    Form5Dot48LowerCharacteristicPrestressingValue,
)
from blueprints.validations import LessOrEqualToZeroError, NegativeValueError


class TestForm5Dot48LowerCharacteristicPrestressingValue:
    """Validation for formula 5.48 from EN 1992-1-1:2004."""

    def test_evaluation(self) -> None:
        """Tests the evaluation of the result."""
        # Example values
        r_inf = 0.95
        p_m_t = 100.0

        # Object to test
        formula = Form5Dot48LowerCharacteristicPrestressingValue(r_inf=r_inf, p_m_t=p_m_t)

        # Expected result, manually calculated
        manually_calculated_result = 95.0

        assert formula == pytest.approx(manually_calculated_result, rel=1e-4)

    @pytest.mark.parametrize(
        ("r_inf", "p_m_t"),
        [
            (-0.95, 100.0),  # r_inf is negative
            (0.95, -100.0),  # p_m_t is negative
        ],
    )
    def test_raise_error_when_invalid_values_are_given(self, r_inf: float, p_m_t: float) -> None:
        """Test invalid values."""
        with pytest.raises((NegativeValueError, LessOrEqualToZeroError)):
            Form5Dot48LowerCharacteristicPrestressingValue(r_inf=r_inf, p_m_t=p_m_t)

    @pytest.mark.parametrize(
        ("representation", "expected"),
        [
            (
                "complete",
                r"P_{k,inf} = r_{inf} \cdot P_{m,t}(x) = 0.950 \cdot 100.000 = 95.000 \ kN",
            ),
            ("short", r"P_{k,inf} = 95.000 \ kN"),
        ],
    )
    def test_latex(self, representation: str, expected: str) -> None:
        """Test the latex representation of the formula."""
        # Example values
        r_inf = 0.95
        p_m_t = 100.0

        # Object to test
        latex = Form5Dot48LowerCharacteristicPrestressingValue(r_inf=r_inf, p_m_t=p_m_t).latex()

        actual = {
            "complete": latex.complete,
            "short": latex.short,
        }

        assert expected == actual[representation], f"{representation} representation failed."



"""Steel Profile Shapes Example
This example demonstrates how to create and visualize different steel profile shapes using the Blueprints library.
"""

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial

# ruff: noqa: T201
from blueprints.structural_sections.steel.steel_cross_sections.chs_profile import CHSSteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.i_profile import ISteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.chs import CHS
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.heb import HEB
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.strip import Strip
from blueprints.structural_sections.steel.steel_cross_sections.strip_profile import StripSteelProfile

# Define steel class
steel_material = SteelMaterial(steel_class=SteelStrengthClass.S355)

# Example usage for CHS profile
chs_profile = CHSSteelProfile.from_standard_profile(
    profile=CHS.CHS273x5,
    steel_material=steel_material,
    corrosion_inside=0,  # mm
    corrosion_outside=4,  # mm
)
chs_profile.plot(show=True)
print(f"Steel class: {chs_profile.steel_material}")
print(f"Moment of inertia about y-axis: {chs_profile.moment_of_inertia_about_y} mm⁴")
print(f"Moment of inertia about z-axis: {chs_profile.moment_of_inertia_about_z} mm⁴")
print(f"Elastic section modulus about y-axis: {chs_profile.elastic_section_modulus_about_y_negative} mm³")
print(f"Elastic section modulus about z-axis: {chs_profile.elastic_section_modulus_about_z_positive} mm³")
print(f"Area: {chs_profile.area} mm²")

# Example usage for custom CHS profile
custom_chs_profile = CHSSteelProfile(
    outer_diameter=150,
    wall_thickness=10,
    steel_material=steel_material,
)
custom_chs_profile.plot(show=True)

# Example usage for Strip profile
strip_profile = StripSteelProfile.from_standard_profile(
    profile=Strip.STRIP160x5,
    steel_material=steel_material,
    corrosion=1,  # mm
)
strip_profile.plot(show=True)

# Example usage for custom Strip profile
custom_strip_profile = StripSteelProfile(
    strip_width=100,
    strip_height=41,
    steel_material=steel_material,
)
custom_strip_profile.plot(show=True)

# Example usage for HEB600 profile
heb_profile = ISteelProfile.from_standard_profile(
    profile=HEB.HEB600,
    steel_material=steel_material,
    corrosion=7,  # mm
)
heb_profile.plot(show=True)

# Example usage for custom I profile
custom_i_profile = ISteelProfile(
    top_flange_width=300,  # mm
    top_flange_thickness=20,  # mm
    bottom_flange_width=200,  # mm
    bottom_flange_thickness=10,  # mm
    total_height=600,  # mm
    web_thickness=10,  # mm
    steel_material=steel_material,
    top_radius=15,  # mm
    bottom_radius=8,  # mm
)
custom_i_profile.plot(show=True)



"""Formula 5.22 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form5Dot22FactorKs(Formula):
    """Class representing the factor K_s that represents the factor for contribution of reinforcement to the resistance."""

    label = "5.22"
    source_document = EN_1992_1_1_2004

    def __init__(self, rho: DIMENSIONLESS) -> None:
        r"""[$K_s$] Factor K_s = 1.

        EN 1992-1-1:2004 art.5.8.7.2(2) - Formula (5.22)

        Parameters
        ----------
        rho : DIMENSIONLESS
            [$\rho$] Geometric reinforcement ratio, As/Ac. Must be >= 0.002.
        """
        super().__init__()
        self.rho = rho

    @staticmethod
    def _evaluate(rho: DIMENSIONLESS) -> float:
        """Evaluates the formula, for more information see the __init__ method."""
        if rho < 0.002:
            raise ValueError(f"Invalid rho: {rho}. rho cannot be less than 0.002")
        return 1.0

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 5.22 K_s."""
        return LatexFormula(
            return_symbol=r"K_s",
            result=f"{self._evaluate(self.rho):.3f}",
            equation=r"1",
            numeric_equation="1",
            comparison_operator_label="=",
        )


class Form5Dot22FactorKc(Formula):
    """Class representing the factor K_c representing the factor for effects of cracking, creep, etc."""

    label = "5.22"
    source_document = EN_1992_1_1_2004

    def __init__(self, k1: DIMENSIONLESS, k2: DIMENSIONLESS, phi_ef: DIMENSIONLESS, rho: float) -> None:
        r"""[$K_c$] Factor K_c = k_1 * k_2 / (1 + \phi_{ef}).

        EN 1992-1-1:2004 art.5.8.7.2(2) - Formula (5.22)

        Parameters
        ----------
        k1 : DIMENSIONLESS
            [$k_1$] Factor which depends on concrete strength class, Expression (5.23).
        k2 : DIMENSIONLESS
            [$k_2$] Factor which depends on axial force and slenderness, Expression (5.24).
        phi_ef : DIMENSIONLESS
            [$\phi_{ef}$] Effective creep ratio, see 5.8.4.
        rho : DIMENSIONLESS
            [$\rho$] Geometric reinforcement ratio, As/Ac. Must be >= 0.002.
        """
        super().__init__()
        self.k1 = k1
        self.k2 = k2
        self.phi_ef = phi_ef
        self.rho = rho

    @staticmethod
    def _evaluate(k1: DIMENSIONLESS, k2: DIMENSIONLESS, phi_ef: DIMENSIONLESS, rho: float) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        if rho < 0.002:
            raise ValueError(f"Invalid rho: {rho}. rho cannot be less than 0.002")
        raise_if_less_or_equal_to_zero(phi_ef=phi_ef)
        raise_if_negative(k1=k1, k2=k2)
        return k1 * k2 / (1 + phi_ef)

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 5.22 K_c."""
        return LatexFormula(
            return_symbol=r"K_c",
            result=f"{self:.3f}",
            equation=r"\frac{k_1 \cdot k_2}{1 + \phi_{ef}}",
            numeric_equation=rf"\frac{{{self.k1:.3f} \cdot {self.k2:.3f}}}{{1 + {self.phi_ef:.3f}}}",
            comparison_operator_label="=",
        )



"""Reinforced concrete cross-section example."""

from blueprints.materials.concrete import ConcreteMaterial, ConcreteStrengthClass
from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial, ReinforcementSteelQuality
from blueprints.structural_sections.concrete.covers import CoversRectangular
from blueprints.structural_sections.concrete.reinforced_concrete_sections.rectangular import RectangularReinforcedCrossSection

# Define a concrete material
concrete = ConcreteMaterial(concrete_class=ConcreteStrengthClass.C35_45)

# Define a reinforcement steel material
steel = ReinforcementSteelMaterial(steel_quality=ReinforcementSteelQuality.B500B)

# Define a rectangular reinforced cross-section
cs = RectangularReinforcedCrossSection(
    width=1000,
    height=800,
    covers=CoversRectangular(upper=45, right=30, lower=35, left=50),
    concrete_material=concrete,
)

# Add reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=14,
    edge="upper",
    material=steel,
)
cs.add_longitudinal_reinforcement_by_quantity(
    n=4,
    diameter=40,
    edge="lower",
    material=steel,
)
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=14,
    edge="right",
    material=steel,
)
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=14,
    edge="left",
    material=steel,
)

# Add stirrups to the cross-section
cs.add_stirrup_along_edges(
    diameter=8,
    distance=150,
    material=steel,
)
# Add stirrups to the cross-section
cs.add_stirrup_along_edges(
    diameter=12,
    distance=300,
    material=steel,
)

cs.plot(show=True)



"""Tests for circular tube cross-section shapes."""

import numpy as np
import pytest

from blueprints.structural_sections.cross_section_tube import TubeCrossSection


class TestTubeCrossSection:
    """Tests for the TubeCrossSection class."""

    def test_area(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the area property of the TubeCrossSection class."""
        expected_area = np.pi * (50.0**2 - 25.0**2)
        assert tube_cross_section.area == pytest.approx(expected=expected_area, rel=1e-6)

    def test_perimeter(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the perimeter property of the TubeCrossSection class."""
        expected_perimeter = 2 * np.pi * 50.0
        assert tube_cross_section.perimeter == pytest.approx(expected=expected_perimeter, rel=1e-6)

    def test_wall_thickness(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the wall thickness property of the TubeCrossSection class."""
        expected_thickness = (100.0 - 50.0) / 2.0
        assert tube_cross_section.wall_thickness == pytest.approx(expected=expected_thickness, rel=1e-6)

    def test_centroid(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the centroid property of the TubeCrossSection class."""
        centroid = tube_cross_section.centroid
        assert (centroid.x, centroid.y) == pytest.approx(expected=(100.0, 250.0), rel=1e-6)

    def test_moments_of_inertia(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the moments of inertia properties of the TubeCrossSection class."""
        expected_inertia = (np.pi / 64) * (100.0**4 - 50.0**4)
        assert tube_cross_section.moment_of_inertia_about_y == pytest.approx(expected=expected_inertia, rel=1e-6)
        assert tube_cross_section.moment_of_inertia_about_z == pytest.approx(expected=expected_inertia, rel=1e-6)

    def test_section_moduli(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the section moduli properties of the TubeCrossSection class."""
        expected_modulus = (np.pi / 64) * (100.0**4 - 50.0**4) / 50.0
        assert tube_cross_section.elastic_section_modulus_about_y_positive == pytest.approx(expected=expected_modulus, rel=1e-6)
        assert tube_cross_section.elastic_section_modulus_about_z_positive == pytest.approx(expected=expected_modulus, rel=1e-6)
        assert tube_cross_section.elastic_section_modulus_about_y_negative == pytest.approx(expected=expected_modulus, rel=1e-6)
        assert tube_cross_section.elastic_section_modulus_about_z_negative == pytest.approx(expected=expected_modulus, rel=1e-6)

    def test_plastic_section_moduli(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the plastic section moduli properties of the TubeCrossSection class."""
        expected_plastic_modulus = (100.0**3 - 50.0**3) / 6
        assert tube_cross_section.plastic_section_modulus_about_y == pytest.approx(expected=expected_plastic_modulus, rel=1e-6)
        assert tube_cross_section.plastic_section_modulus_about_z == pytest.approx(expected=expected_plastic_modulus, rel=1e-6)

    def test_polygon(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the geometry property of the TubeCrossSection class."""
        polygon = tube_cross_section.polygon
        assert polygon.is_valid
        assert polygon.area == pytest.approx(expected=tube_cross_section.area, rel=1e-3)

    def test_invalid_outer_diameter(self) -> None:
        """Test initialization with an invalid outer diameter value."""
        with pytest.raises(ValueError, match="Outer diameter must be a positive value"):
            TubeCrossSection(name="InvalidOuter", outer_diameter=-10.0, inner_diameter=5.0, x=0.0, y=0.0)

    def test_invalid_inner_diameter(self) -> None:
        """Test initialization with an invalid inner diameter value."""
        with pytest.raises(ValueError, match="Inner diameter cannot be negative"):
            TubeCrossSection(name="InvalidInner", outer_diameter=10.0, inner_diameter=-5.0, x=0.0, y=0.0)

    def test_inner_diameter_greater_than_outer(self) -> None:
        """Test initialization with inner diameter greater than or equal to outer diameter."""
        with pytest.raises(ValueError, match="Inner diameter must be smaller than outer diameter"):
            TubeCrossSection(name="InvalidDiameters", outer_diameter=9.0, inner_diameter=10.0, x=0.0, y=0.0)

    def test_geometry(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the geometry property of the TubeCrossSection class."""
        geometry = tube_cross_section.geometry()
        assert geometry is not None

    def test_section_properties(self, tube_cross_section: TubeCrossSection) -> None:
        """Test the section properties of the TubeCrossSection class."""
        section_properties = tube_cross_section.section_properties()
        assert section_properties.area == pytest.approx(expected=tube_cross_section.area, rel=1e-2)
        assert section_properties.perimeter == pytest.approx(expected=tube_cross_section.perimeter, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=tube_cross_section.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=tube_cross_section.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=tube_cross_section.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=tube_cross_section.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=tube_cross_section.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=tube_cross_section.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=tube_cross_section.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=tube_cross_section.elastic_section_modulus_about_z_negative, rel=1e-2)
        assert section_properties.sxx == pytest.approx(expected=tube_cross_section.plastic_section_modulus_about_y, rel=1e-2)
        assert section_properties.syy == pytest.approx(expected=tube_cross_section.plastic_section_modulus_about_z, rel=1e-2)



"""Formula 9.10 from EN 1992-1-1:2004: Chapter 9 - Detailling and specific rules."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form9Dot10MaximumSpacingBentUpBars(Formula):
    """Class representing the formula 9.10 for the calculation of the maximum longitudinal spacing of bent up bars for slabs."""

    label = "9.10"
    source_document = EN_1992_1_1_2004

    def __init__(self, d: MM) -> None:
        r"""[$s_{max}$] Maximum longitudinal spacing of bent up bars for slabs [$mm$].

        EN 1992-1-1:2004 art.9.3.2(4) - Formula (9.10)

        Parameters
        ----------
        d: MM
            [$d$] Effective height of the cross-section [$mm$].
        """
        super().__init__()
        self.d = d

    @staticmethod
    def _evaluate(d: MM) -> MM:
        """For more detailed documentation see the class docstring."""
        raise_if_negative(d=d)
        return d

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 9.10."""
        return LatexFormula(
            return_symbol=r"s_{max}",
            result=f"{self:.2f}",
            equation=r"d",
            numeric_equation=rf"{self.d:.2f}",
            comparison_operator_label="=",
        )



"""Test suite for StripSteelProfile."""

import matplotlib.pyplot as plt
import pytest

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.strip import Strip
from blueprints.structural_sections.steel.steel_cross_sections.strip_profile import StripSteelProfile


class TestStripSteelProfile:
    """Test suite for StripSteelProfile."""

    def test_code(self, strip_profile: StripSteelProfile) -> None:
        """Test the code of the Strip profile."""
        expected_alias = "160x5"
        assert strip_profile.name == expected_alias

    def test_steel_volume_per_meter(self, strip_profile: StripSteelProfile) -> None:
        """Test the steel volume per meter."""
        expected_volume = 0.160 * 0.005  # m³/m
        assert pytest.approx(strip_profile.volume_per_meter, rel=1e-6) == expected_volume

    def test_steel_weight_per_meter(self, strip_profile: StripSteelProfile) -> None:
        """Test the steel weight per meter."""
        expected_weight = 0.160 * 0.005 * 7850  # kg/m
        assert pytest.approx(strip_profile.weight_per_meter, rel=1e-6) == expected_weight

    def test_area(self, strip_profile: StripSteelProfile) -> None:
        """Test the steel cross-sectional area."""
        expected_area = 160 * 5  # mm²
        assert pytest.approx(strip_profile.area, rel=1e-6) == expected_area

    def test_centroid(self, strip_profile: StripSteelProfile) -> None:
        """Test the centroid of the steel cross-section."""
        expected_centroid = (0, 0)  # (x, y) coordinates
        assert pytest.approx(strip_profile.centroid.x, rel=1e-6) == expected_centroid[0]
        assert pytest.approx(strip_profile.centroid.y, rel=1e-6) == expected_centroid[1]

    def test_moment_of_inertia_about_y(self, strip_profile: StripSteelProfile) -> None:
        """Test the moment of inertia about the y-axis."""
        expected_moi_y = 1 / 12 * 160 * 5**3  # mm⁴
        assert pytest.approx(strip_profile.moment_of_inertia_about_y, rel=1e-6) == expected_moi_y

    def test_moment_of_inertia_about_z(self, strip_profile: StripSteelProfile) -> None:
        """Test the moment of inertia about the z-axis."""
        expected_moi_z = 1 / 12 * 160**3 * 5  # mm⁴
        assert pytest.approx(strip_profile.moment_of_inertia_about_z, rel=1e-6) == expected_moi_z

    def test_elastic_section_modulus_about_y_positive(self, strip_profile: StripSteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the positive z side."""
        expected_modulus_y_positive = 1 / 6 * 160 * 5**2  # mm³
        assert pytest.approx(strip_profile.elastic_section_modulus_about_y_positive, rel=1e-6) == expected_modulus_y_positive

    def test_elastic_section_modulus_about_y_negative(self, strip_profile: StripSteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the negative z side."""
        expected_modulus_y_negative = 1 / 6 * 160 * 5**2  # mm³
        assert pytest.approx(strip_profile.elastic_section_modulus_about_y_negative, rel=1e-6) == expected_modulus_y_negative

    def test_elastic_section_modulus_about_z_positive(self, strip_profile: StripSteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the positive y side."""
        expected_modulus_z_positive = 1 / 6 * 160**2 * 5  # mm³
        assert pytest.approx(strip_profile.elastic_section_modulus_about_z_positive, rel=1e-6) == expected_modulus_z_positive

    def test_elastic_section_modulus_about_z_negative(self, strip_profile: StripSteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the negative y side."""
        expected_modulus_z_negative = 1 / 6 * 160**2 * 5  # mm³
        assert pytest.approx(strip_profile.elastic_section_modulus_about_z_negative, rel=1e-6) == expected_modulus_z_negative

    def test_plot(self, strip_profile: StripSteelProfile) -> None:
        """Test the plot method (ensure it runs without errors)."""
        fig = strip_profile.plot(show=False)
        assert isinstance(fig, plt.Figure)

    def test_geometry(self, strip_profile: StripSteelProfile) -> None:
        """Test the geometry of the Strip profile."""
        expected_geometry = strip_profile.geometry
        assert expected_geometry is not None

    def test_section_properties(self, strip_profile: StripSteelProfile) -> None:
        """Test the section properties of the Strip profile."""
        section_properties = strip_profile.section_properties()
        assert section_properties.mass == pytest.approx(expected=strip_profile.area, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=strip_profile.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=strip_profile.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=strip_profile.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=strip_profile.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=strip_profile.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=strip_profile.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=strip_profile.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=strip_profile.elastic_section_modulus_about_z_negative, rel=1e-2)

    def test_yield_strength(self, strip_profile: StripSteelProfile) -> None:
        """Test the yield strength of the Strip profile."""
        assert strip_profile.yield_strength == 355

    def test_ultimate_strength(self, strip_profile: StripSteelProfile) -> None:
        """Test the ultimate strength of the Strip profile."""
        assert strip_profile.ultimate_strength == 490

    def test_plastic_section_modulus_about_y(self, strip_profile: StripSteelProfile) -> None:
        """Test the plastic section modulus about the y-axis."""
        expected_plastic_modulus_y = 1 / 4 * 160 * 5**2
        assert pytest.approx(strip_profile.plastic_section_modulus_about_y, rel=1e-6) == expected_plastic_modulus_y

    def test_plastic_section_modulus_about_z(self, strip_profile: StripSteelProfile) -> None:
        """Test the plastic section modulus about the z-axis."""
        expected_plastic_modulus_z = 1 / 4 * 5 * 160**2
        assert pytest.approx(strip_profile.plastic_section_modulus_about_z, rel=1e-6) == expected_plastic_modulus_z

    def test_get_profile_with_corrosion(self) -> None:
        """Test the Strip profile with 2 mm corrosion applied."""
        # Ensure the profile raises an error if fully corroded
        with pytest.raises(ValueError, match="The profile has fully corroded."):
            StripSteelProfile.from_standard_profile(
                profile=Strip.STRIP160x5,
                steel_material=SteelMaterial(SteelStrengthClass.S355),
                corrosion=2.5,
            )

    def test_corrosion_in_name(self, strip_profile: StripSteelProfile) -> None:
        """Test that the corrosion is included in the profile name."""
        profile_with_corrosion = StripSteelProfile.from_standard_profile(
            profile=Strip.STRIP160x5,
            steel_material=SteelMaterial(SteelStrengthClass.S355),
            corrosion=1,
        )
        expected_name = f"{strip_profile.name} (corrosion: 1 mm)"
        assert profile_with_corrosion.name == expected_name



"""Formula 8.12 from EN 1992-1-1:2004: Chapter 8: Detailing of reinforcement and prestressing tendons."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MM2
from blueprints.validations import raise_if_negative


class Form8Dot12AdditionalShearReinforcement(Formula):
    """Class representing formula 8.12 for the calculation of the minimum additional shear reinforcement in the anchorage zones where transverse
    compression is not present for straight anchorage lengths, in the direction parallel to the tension face.
    """

    label = "8.12"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        a_s: MM2,
        n_1: DIMENSIONLESS,
    ) -> None:
        r"""[$A_{sh}$] Minimum additional shear reinforcement in the anchorage zones where transverse compression is not present for straight
        anchorage lengths, in the direction parallel to the tension face [$mm^2$].

        EN 1992-1-1:2004 art.8.8(6) - Formula (8.12)

        Parameters
        ----------
        a_s: MM2
            [$A_{s}$] Cross sectional area of reinforcement [$mm²$].
        n_1: DIMENSIONLESS
            [$n_{1}$] Number of layers with bars anchored at the same point in the member [$-$].
        """
        super().__init__()
        self.a_s = a_s
        self.n_1 = n_1

    @staticmethod
    def _evaluate(
        a_s: MM2,
        n_1: DIMENSIONLESS,
    ) -> MM2:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(a_s=a_s, n_1=n_1)
        return 0.25 * a_s * n_1

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 8.12."""
        return LatexFormula(
            return_symbol=r"A_{sh}",
            result=f"{self:.2f}",
            equation=r"0.25 \cdot A_s \cdot n_1",
            numeric_equation=rf"0.25 \cdot {self.a_s:.2f} \cdot {self.n_1:.2f}",
            comparison_operator_label="=",
        )



"""Formula 6.37 from EN 1992-1-1:2004: Chapter 6 - Ultimate limit state."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM
from blueprints.validations import raise_if_negative


class Form6Dot37InternalContourRadiusCircularColumnHeads(Formula):
    r"""Class representing formula 6.37 for the calculation of the contour radius for circular column heads, [$r_{cont,int}$]."""

    label = "6.37"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        d: MM,
        h_h: MM,
        c: MM,
    ) -> None:
        r"""[$r_{cont,int}$] Contour radius for circular column heads [$mm$].

        EN 1992-1-1:2004 art.6.4.2(11) - Formula (6.37)

        Parameters
        ----------
        d : MM
            [$d$] Effective depth [$mm$].
        h_h : MM
            [$h_{H}$] height of the column head [$mm$].
        c : MM
            [$c$] Diameter of a circular column [$mm$].
        """
        super().__init__()
        self.d = d
        self.h_h = h_h
        self.c = c

    @staticmethod
    def _evaluate(
        d: MM,
        h_h: MM,
        c: MM,
    ) -> MM:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(d=d, h_h=h_h, c=c)
        return 2 * (d + h_h) + 0.5 * c

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 6.37."""
        return LatexFormula(
            return_symbol=r"r_{cont,int}",
            result=f"{self:.3f}",
            equation=r"2 \cdot (d + h_{H}) + 0.5 \cdot c",
            numeric_equation=rf"2 \cdot ({self.d:.3f} + {self.h_h:.3f}) + 0.5 \cdot {self.c:.3f}",
            comparison_operator_label="=",
            unit="mm",
        )



"""Tests for hexagonal cross-section shapes."""

import numpy as np
import pytest

from blueprints.structural_sections.cross_section_hexagon import HexagonalCrossSection


class TestHexagonalCrossSection:
    """Tests for the HexagonalCrossSection class."""

    def test_area(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the area property of the HexagonalCrossSection class."""
        expected_area = (3 * np.sqrt(3) / 2) * 50.0**2
        assert hexagonal_cross_section.area == pytest.approx(expected=expected_area, rel=1e-6)

    def test_perimeter(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the perimeter property of the HexagonalCrossSection class."""
        expected_perimeter = 6 * 50.0
        assert hexagonal_cross_section.perimeter == pytest.approx(expected=expected_perimeter, rel=1e-6)

    def test_centroid(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the centroid property of the HexagonalCrossSection class."""
        centroid = hexagonal_cross_section.centroid
        assert (centroid.x, centroid.y) == pytest.approx(expected=(100.0, 250.0), rel=1e-6)

    def test_moments_of_inertia(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the moments of inertia properties of the HexagonalCrossSection class."""
        expected_inertia = (5 / 16) * np.sqrt(3) * 50.0**4
        assert hexagonal_cross_section.moment_of_inertia_about_y == pytest.approx(expected=expected_inertia, rel=1e-6)
        assert hexagonal_cross_section.moment_of_inertia_about_z == pytest.approx(expected=expected_inertia, rel=1e-6)

    def test_section_moduli(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the section moduli properties of the HexagonalCrossSection class."""
        expected_modulus_y = ((5 / 16) * np.sqrt(3) * 50.0**4) / (50.0 * np.sqrt(3) / 2)
        expected_modulus_z = ((5 / 16) * np.sqrt(3) * 50.0**4) / 50.0
        assert hexagonal_cross_section.elastic_section_modulus_about_y_positive == pytest.approx(expected=expected_modulus_y, rel=1e-6)
        assert hexagonal_cross_section.elastic_section_modulus_about_z_positive == pytest.approx(expected=expected_modulus_z, rel=1e-6)
        assert hexagonal_cross_section.elastic_section_modulus_about_y_negative == pytest.approx(expected=expected_modulus_y, rel=1e-6)
        assert hexagonal_cross_section.elastic_section_modulus_about_z_negative == pytest.approx(expected=expected_modulus_z, rel=1e-6)

    def test_plastic_section_moduli(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the plastic section moduli properties of the HexagonalCrossSection class."""
        assert hexagonal_cross_section.plastic_section_modulus_about_y == pytest.approx(expected=50.0**3, rel=1e-6)
        expected = 2 * (50 / 4 * (50 / 2) * (50 * np.sqrt(3)) + (50 * 2 / 3) * (0.5 * (50 * np.sqrt(3)) * (50 / 2)))
        assert hexagonal_cross_section.plastic_section_modulus_about_z == pytest.approx(expected=expected, rel=1e-6)

    def test_polygon(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the geometry property of the HexagonalCrossSection class."""
        polygon = hexagonal_cross_section.polygon
        assert polygon.is_valid
        assert polygon.area == pytest.approx(expected=hexagonal_cross_section.area, rel=1e-4)

    def test_invalid_side_length(self) -> None:
        """Test initialization with an invalid side length value."""
        with pytest.raises(ValueError, match="Side length must be a positive value"):
            HexagonalCrossSection(name="InvalidHexagon", side_length=-10.0, x=0.0, y=0.0)

    def test_geometry(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the geometry property of the HexagonalCrossSection class."""
        geometry = hexagonal_cross_section.geometry()
        assert geometry is not None

    def test_section_properties(self, hexagonal_cross_section: HexagonalCrossSection) -> None:
        """Test the section properties of the HexagonalCrossSection class."""
        section_properties = hexagonal_cross_section.section_properties()
        assert section_properties.area == pytest.approx(expected=hexagonal_cross_section.area, rel=1e-2)
        assert section_properties.perimeter == pytest.approx(expected=hexagonal_cross_section.perimeter, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=hexagonal_cross_section.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=hexagonal_cross_section.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=hexagonal_cross_section.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=hexagonal_cross_section.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=hexagonal_cross_section.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=hexagonal_cross_section.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=hexagonal_cross_section.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=hexagonal_cross_section.elastic_section_modulus_about_z_negative, rel=1e-2)
        assert section_properties.sxx == pytest.approx(expected=hexagonal_cross_section.plastic_section_modulus_about_y, rel=1e-2)
        assert section_properties.syy == pytest.approx(expected=hexagonal_cross_section.plastic_section_modulus_about_z, rel=1e-2)



"""Formula 8.15 from EN 1992-1-1:2004: Chapter 8: Detailing of reinforcement and prestressing tendons."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form8Dot15PrestressTransferStress(Formula):
    """Class representing formula 8.15 for the calculation of the constant bond stress at which prestress is assumed to be transferred to the
    concrete, at the release of tendons.
    """

    label = "8.15"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        eta_p1: DIMENSIONLESS,
        eta_1: DIMENSIONLESS,
        f_ctd_t: MPA,
    ) -> None:
        r"""[$f_{bpt}$] Constant bond stress at which prestress is assumed to be transferred to the concrete [$MPa$].
        EN 1992-1-1:2004 art.8.10.2.2(1) - Formula (8.15).

        Parameters
        ----------
        eta_p1 : DIMENSIONLESS
            [$\eta_{p1}$] Coefficient that takes into account the type of tendon and the bond situation at release [$-$].
            = 2.7 for indented wires.
            = 3.2 for 3 and 7-wire strands
            Use your own implementation for this value or use :class:`SubForm8Dot15EtaP1` class.
        eta_1 : DIMENSIONLESS
            [$\eta_1$] Coefficient related to the quality of the bond condition and the position of bar during concreting (see Figure 8.2) [$-$].
            = 1 when 'good' conditions are obtained;
            = 0.7 other cases and for bars in structural elements built with slip-forms, unless it can be shown that 'good' bond conditions exist;
            Use your own implementation of this formula or use the :class:`SubForm8Dot2CoefficientQualityOfBond` class.
        f_ctd_t : MPA
            [$f_{ctd}(t)$] Design tensile value of strength at time of release [$MPa$].
            = [$\alpha_{ct} \cdot 0.7 \cdot f_{ctm}(t) / \gamma_{c}$] (see 3.1.2(9) and 3.1.6(2)P)
            Use your own implementation for this value or use :class:`SubForm8Dot15TensileStrengthAtRelease` class.
        """
        super().__init__()
        self.eta_p1 = eta_p1
        self.eta_1 = eta_1
        self.f_ctd_t = f_ctd_t

    @staticmethod
    def _evaluate(
        eta_p1: DIMENSIONLESS,
        eta_1: DIMENSIONLESS,
        f_ctd_t: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            eta_p1=eta_p1,
            eta_1=eta_1,
            f_ctd_t=f_ctd_t,
        )
        return eta_p1 * eta_1 * f_ctd_t

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 8.15."""
        return LatexFormula(
            return_symbol=r"f_{bpt}",
            result=f"{self:.2f}",
            equation=r"\eta_{p1} \cdot \eta_1 \cdot f_{ctd}(t)",
            numeric_equation=rf"{self.eta_p1:.2f} \cdot {self.eta_1:.2f} \cdot {self.f_ctd_t:.2f}",
            comparison_operator_label="=",
        )


class SubForm8Dot15EtaP1(Formula):
    """Class representing sub-formula 8.15 for the calculation of the coefficient that takes into account the type of tendon and the bond situation
    at release.
    """

    label = "8.15"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        type_of_wire: str,
    ) -> None:
        r"""[$\eta_{p1}$] Coefficient that takes into account the type of tendon and the bond situation at release [$-$].

        EN 1992-1-1:2004 art.8.10.2.2(1) - Formula (8.15)

        Parameters
        ----------
        type_of_wire : str
            Type of wire.

            = 'indented' for indented wires;

            = '3_7_wire_strands' for 3 and 7-wire strands;
        """
        super().__init__()
        self.type_of_wire = type_of_wire

    @staticmethod
    def _evaluate(type_of_wire: str) -> DIMENSIONLESS:
        """Evaluates the formula, for more information see the __init__ method."""
        match type_of_wire.lower():
            case "indented":
                return 2.7
            case "3_7_wire_strands":
                return 3.2
            case _:
                raise ValueError(f"Invalid type of wire: {type_of_wire}. Options: 'indented' or '3_7_wire_strands'")

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for the first subformula of formula 8.15."""
        return LatexFormula(
            return_symbol=r"\eta_{p1}",
            result=f"{self:.2f}",
            equation=r"type\;of\;wire",
            numeric_equation=f"{self.type_of_wire}".replace(" ", r"\;"),
            comparison_operator_label=r"\rightarrow",
        )


class SubForm8Dot15TensileStrengthAtRelease(Formula):
    """Class representing sub-formula 8.15 for the calculation of the design tensile value of strength at time of release (see  3.1.2(8) and
    3.1.6(2)P).
    """

    label = "8.15"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        alpha_ct: DIMENSIONLESS,
        f_ctm_t: MPA,
        gamma_c: DIMENSIONLESS,
    ) -> None:
        r"""[$f_{ctd}(t)$] Design tensile value of strength at time of release [$MPa$].

        EN 1992-1-1:2004 art.8.10.2.2(1) - Formula (8.15)

        Parameters
        ----------
        alpha_ct : DIMENSIONLESS
            [$\alpha_{ct}$] coefficient taking account of long term effects on the tensile strength and of unfavourable effects, resulting from the
            way the load is applied. [$-$].

            Value may be found in national annex. Recommended value: 1.0
        f_ctm_t : MPA
            [$f_{ctm}(t)$] Mean value of tensile strength at time of release (see formula 3.4) [$MPa$].

            Use your own implementation for this value or use :class:`Form3Dot4DevelopmentTensileStrength` class.
        gamma_c : DIMENSIONLESS
            [$\gamma_{c}$] Partial safety factor for concrete [$-$].
        """
        super().__init__()
        self.alpha_ct = alpha_ct
        self.f_ctm_t = f_ctm_t
        self.gamma_c = gamma_c

    @staticmethod
    def _evaluate(
        alpha_ct: DIMENSIONLESS,
        f_ctm_t: MPA,
        gamma_c: DIMENSIONLESS,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(alpha_ct=alpha_ct, f_ctm_t=f_ctm_t)
        raise_if_less_or_equal_to_zero(gamma_c=gamma_c)
        return alpha_ct * 0.7 * f_ctm_t / gamma_c

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for the second subformula of formula 8.15."""
        return LatexFormula(
            return_symbol=r"f_{ctd}(t)",
            result=f"{self:.2f}",
            equation=r"\frac{\alpha_{ct} \cdot 0.7 \cdot f_{ctm}(t)}{\gamma_c}",
            numeric_equation=rf"\frac{{{self.alpha_ct:.2f} \cdot 0.7 \cdot {self.f_ctm_t:.2f}}}{{{self.gamma_c:.2f}}}",
            comparison_operator_label="=",
        )



"""Reinforced concrete cross-section example."""

from shapely import LineString

from blueprints.materials.concrete import ConcreteMaterial, ConcreteStrengthClass
from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial, ReinforcementSteelQuality
from blueprints.structural_sections.concrete.rebar import Rebar
from blueprints.structural_sections.concrete.reinforced_concrete_sections.circular import CircularReinforcedCrossSection, ReinforcementByQuantity

# Define a concrete material
concrete = ConcreteMaterial(concrete_class=ConcreteStrengthClass.C35_45)

# Define a reinforcement steel material
steel = ReinforcementSteelMaterial(steel_quality=ReinforcementSteelQuality.B500B)

# Define a circular reinforced cross-section
cs = CircularReinforcedCrossSection(
    diameter=400,
    cover=35,
    concrete_material=concrete,
)

# Add longitudinal reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=25,
    material=steel,
)

# Add longitudinal reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=16,
    material=steel,
    start_angle=45,
)

# Add stirrups to the cross-section
cs.add_stirrup_along_perimeter(
    diameter=10,
    distance=150,
    material=steel,
)

# Add reinforcement configuration to the cross-section in a diagonal line
linestring = LineString([(-100, 100), (100, 0)])
cs.add_reinforcement_configuration(
    line=linestring,
    configuration=ReinforcementByQuantity(
        diameter=12,
        n=7,
        material=steel,
    ),
)

# Add longitudinal rebar in center
cs.add_longitudinal_rebar(rebar=Rebar(diameter=16, x=0, y=0, material=steel))

# Plot the cross-section
cs.plot(show=True)



"""Formula 6.9 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.math_helpers import cot
from blueprints.type_alias import DEG, DIMENSIONLESS, MM, MPA, N
from blueprints.validations import raise_if_less_or_equal_to_zero, raise_if_negative


class Form6Dot9MaximumShearResistance(Formula):
    r"""Class representing formula 6.9 for the calculation of the maximum shear resistance, $V_{Rd,max}$."""

    label = "6.9"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        b_w: MM,
        z: MM,
        f_cd: MPA,
        nu_1: DIMENSIONLESS,
        alpha_cw: DIMENSIONLESS,
        theta: DEG,
    ) -> None:
        r"""[$V_{Rd,max}$] Maximum shear resistance [$N$].

        EN 1992-1-1:2004 art.6.2.3(3) - Formula (6.9)

        Parameters
        ----------
        b_w : MM
            [$b_{w}$] Width of the web of the beam [$mm$].
        z : MM
            [$z$] Lever arm [$mm$].
        f_cd : MPA
            [$f_{cd}$] Design value of concrete compressive strength [$MPa$].
        nu_1 : DIMENSIONLESS
            [$\nu_{1}$] Strength reduction factor for concrete cracked in shear [-].
        alpha_cw : DIMENSIONLESS
            [$\alpha_{cw}$] Coefficient taking account of the state of the stress in the compression chord [-].
        theta : DEG
            [$\theta$] Angle between the concrete compression strut and the beam axis perpendicular to the shear force [$degrees$].
        """
        super().__init__()
        self.b_w = b_w
        self.z = z
        self.f_cd = f_cd
        self.nu_1 = nu_1
        self.alpha_cw = alpha_cw
        self.theta = theta

    @staticmethod
    def _evaluate(
        b_w: MM,
        z: MM,
        f_cd: MPA,
        nu_1: DIMENSIONLESS,
        alpha_cw: DIMENSIONLESS,
        theta: DEG,
    ) -> N:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            b_w=b_w,
            z=z,
            f_cd=f_cd,
            nu_1=nu_1,
            alpha_cw=alpha_cw,
        )
        raise_if_less_or_equal_to_zero(theta=theta)

        return alpha_cw * b_w * z * nu_1 * f_cd / (cot(theta) + np.tan(np.deg2rad(theta)))

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 6.9."""
        return LatexFormula(
            return_symbol=r"V_{Rd,max}",
            result=f"{self:.3f}",
            equation=r"\alpha_{cw} \cdot b_{w} \cdot z \cdot \nu_{1} \cdot \frac{f_{cd}}{\cot(\theta) + \tan(\theta)}",
            numeric_equation=rf"{self.alpha_cw:.3f} \cdot {self.b_w:.3f} \cdot {self.z:.3f} \cdot {self.nu_1:.3f} \cdot"
            rf" \frac{{{self.f_cd:.3f}}}{{\cot({self.theta:.3f}) + \tan({self.theta:.3f})}}",
            comparison_operator_label="=",
            unit="N",
        )



"""Custom rectangular reinforced cross-section example."""

from shapely import LineString, Polygon

from blueprints.materials.concrete import ConcreteMaterial, ConcreteStrengthClass
from blueprints.materials.reinforcement_steel import ReinforcementSteelMaterial, ReinforcementSteelQuality
from blueprints.structural_sections.concrete.covers import CoversRectangular
from blueprints.structural_sections.concrete.rebar import Rebar
from blueprints.structural_sections.concrete.reinforced_concrete_sections.rectangular import RectangularReinforcedCrossSection
from blueprints.structural_sections.concrete.reinforced_concrete_sections.reinforcement_configurations import (
    ReinforcementByDistance,
    ReinforcementByQuantity,
)
from blueprints.structural_sections.concrete.stirrups import StirrupConfiguration

# Define a concrete material
concrete = ConcreteMaterial(concrete_class=ConcreteStrengthClass.C30_37)

# Define a reinforcement steel material
steel = ReinforcementSteelMaterial(steel_quality=ReinforcementSteelQuality.B500B)

# Define a rectangular reinforced cross-section
cs = RectangularReinforcedCrossSection(
    width=600,
    height=500,
    covers=CoversRectangular(upper=45, right=30, lower=35, left=50),
    concrete_material=concrete,
)

# Add reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=5,
    diameter=14,
    edge="upper",
    material=steel,
)

# add a second layer of reinforcement to the cross-section
cs.add_longitudinal_reinforcement_by_quantity(
    n=3,
    diameter=16,
    edge="upper",
    material=steel,
    cover=100,
)

# add reinforcement configurations to the cross-section in any position
cs.add_reinforcement_configuration(
    line=LineString([(50, -100), (150, 50)]),
    configuration=ReinforcementByQuantity(diameter=20, n=3, material=steel),
)

cs.add_reinforcement_configuration(
    line=LineString([(0, -180), (-250, 0)]),
    configuration=ReinforcementByDistance(diameter=12, center_to_center=40, material=steel),
)

# Add stirrups to the cross-section
cs.add_stirrup_configuration(
    stirrup=StirrupConfiguration(
        geometry=Polygon([(-200, -200), (-200, 200), (200, 200), (200, -200)]),
        diameter=8,
        distance=150,
        material=steel,
    ),
)

# Add a longitudinal rebar to the cross-section
cs.add_longitudinal_rebar(
    rebar=Rebar(
        diameter=12,
        x=0,
        y=0,
        material=steel,
    )
)

cs.plot(show=True)



"""Formula 3.18 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MPA


class Form3Dot18CompressiveStressConcrete(Formula):
    """Class representing formula 3.18 for the calculation of compressive stress in concrete using stress-strain diagram of figure 3.3."""

    label = "3.18"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_cd: MPA,
    ) -> None:
        r"""[$\sigma_c$] Compressive stress in concrete using stress-strain diagram of figure 3.3 [$MPa$].

        EN 1992-1-1:2004 art.3.1.7(1) - Formula (3.18)

        Parameters
        ----------
        f_cd : MPA
            [$f_{cd}$] Design value compressive strength concrete [$MPa$].

        Returns
        -------
        None
        """
        super().__init__()
        self.f_cd = f_cd

    @staticmethod
    def _evaluate(
        f_cd: MPA,
    ) -> MPA:
        """Evaluates the formula, for more information see the __init__ method."""
        if f_cd < 0:
            raise ValueError(f"Invalid f_cd: {f_cd}. f_cd cannot be negative")
        return f_cd

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 3.18."""
        return LatexFormula(
            return_symbol=r"\sigma_c",
            result=f"{self:.3f}",
            equation=r"f_{cd}",
            numeric_equation=rf"{self.f_cd:.3f}",
            comparison_operator_label="=",
        )



"""Formula 3.27 from EN 1992-1-1:2004: Chapter 3 - Materials."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, MPA


class Form3Dot27IncreasedStrainLimitValue(Formula):
    """Class representing formula 3.27 for the calculation of the increased strain limit value due to enclosed concrete."""

    label = "3.27"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        f_ck: MPA,
        sigma_2: MPA,
        epsilon_cu2: DIMENSIONLESS,
    ) -> None:
        r"""[$\epsilon_{cu2,c}$] Increased strain limit value due to enclosed concrete. [$-$].

        EN 1992-1-1:2004 art.3.1.9(2) - Formula (3.27)

        Parameters
        ----------
        f_ck : MPA
            [$f_{ck}$] Characteristic compressive strength [$MPa$]
        sigma_2 : MPA
            [$\sigma_{2}$] Effective compressive stress in transverse direction [$MPa$]
        epsilon_cu2 : DIMENSIONLESS
            [$\epsilon_{cu2}$] Strain limit value [$-$]

        Returns
        -------
        None
        """
        super().__init__()
        self.f_ck = f_ck
        self.sigma_2 = sigma_2
        self.epsilon_cu2 = epsilon_cu2

    @staticmethod
    def _evaluate(
        f_ck: MPA,
        sigma_2: MPA,
        epsilon_cu2: DIMENSIONLESS,
    ) -> float:
        """Evaluates the formula, for more information see the __init__ method."""
        if f_ck < 0:
            raise ValueError(f"Invalid f_ck: {f_ck}. f_ck cannot be negative")
        return epsilon_cu2 + 0.2 * sigma_2 / f_ck

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 3.27."""
        return LatexFormula(
            return_symbol=r"\epsilon_{cu2,c}",
            result=f"{self:.3f}",
            equation=r"\epsilon_{cu2} + 0.2 \cdot \sigma_2 / f_{ck}",
            numeric_equation=rf"{self.epsilon_cu2:.3f} + 0.2 \cdot {self.sigma_2:.3f} / {self.f_ck:.3f}",
            comparison_operator_label="=",
        )



"""Formula 5.3a from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN
from blueprints.validations import raise_if_negative


class Form5Dot3aTransverseForceUnbracedMembers(Formula):
    """Class representing formula 5.3a for the calculation of the transverse force for unbraced members, [$H_{i}$].

    See Figure 5.1 a1.
    """

    label = "5.3a"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        theta_i: DIMENSIONLESS,
        n_axial_force: KN,
    ) -> None:
        r"""[$H_{i}$] Transverse force for unbraced members [$kN$].

        EN 1992-1-1:2004 art.5.2(7) - Formula (5.3a)

        Parameters
        ----------
        theta_i : DIMENSIONLESS
            [$\Theta_{i}$] Eccentricity, initial inclination imperfections [-].

            Use your own implementation of this value or use the :class:`Form5Dot1Imperfections` class.
        n_axial_force : KN
            [$N$] Axial force [$kN$].

            Positive values for compression, tension is not allowed.

        Notes
        -----
        Eccentricity is suitable for statically determinate members, whereas transverse load can be used for
        both determinate and indeterminate members. The force [$H_{i}$] may be substituted by some other equivalent
        transverse action.
        """
        super().__init__()
        self.theta_i = theta_i
        self.n_axial_force = n_axial_force

    @staticmethod
    def _evaluate(
        theta_i: DIMENSIONLESS,
        n_axial_force: KN,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(theta_i=theta_i, n_axial_force=n_axial_force)
        return theta_i * n_axial_force

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 5.3a."""
        return LatexFormula(
            return_symbol=r"H_i",
            result=f"{self:.3f}",
            equation=r"\theta_i \cdot N",
            numeric_equation=rf"{self.theta_i:.3f} \cdot {self.n_axial_force:.3f}",
            comparison_operator_label="=",
        )



"""Formula 6.41 from EN 1992-1-1:2004: Chapter 6 - Ultimate Limit State."""

import numpy as np

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import MM, MM2
from blueprints.validations import raise_if_negative


class Form6Dot41W1Rectangular(Formula):
    r"""Class representing formula 6.41 for the calculation of [$W_1$]."""

    label = "6.41"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        c_1: MM,
        c_2: MM,
        d: MM,
    ) -> None:
        r"""[$W_1$] Calculation of [$W_1$].

        EN 1992-1-1:2004 art.6.4.3(3) - Formula (6.41)

        Parameters
        ----------
        c_1 : MM
            [$c_1$] Column dimension parallel to the eccentricity of the load [$mm$].
        c_2 : MM
            [$c_2$] Column dimension perpendicular to the eccentricity of the load [$mm$].
        d : MM
            [$d$] Mean effective depth of the slab [$mm$].
        """
        super().__init__()
        self.c_1 = c_1
        self.c_2 = c_2
        self.d = d

    @staticmethod
    def _evaluate(
        c_1: MM,
        c_2: MM,
        d: MM,
    ) -> MM2:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(c_1=c_1, c_2=c_2, d=d)

        return (c_1**2) / 2 + c_1 * c_2 + 4 * c_2 * d + 16 * d**2 + 2 * np.pi * d * c_1

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 6.41."""
        return LatexFormula(
            return_symbol=r"W_1",
            result=f"{self:.3f}",
            equation=r"\frac{c_1^2}{2} + c_1 \cdot c_2 + 4 \cdot c_2 \cdot d + 16 \cdot d^2 + 2 \cdot \pi \cdot d \cdot c_1",
            numeric_equation=rf"\frac{{{self.c_1:.3f}^2}}{{2}} + {self.c_1:.3f} \cdot "
            rf"{self.c_2:.3f} + 4 \cdot {self.c_2:.3f} \cdot {self.d:.3f} + 16 \cdot {self.d:.3f}^2 + "
            rf"2 \cdot \pi \cdot {self.d:.3f} \cdot {self.c_1:.3f}",
            comparison_operator_label="=",
            unit="mm^2",
        )



"""Circular cross-section shape."""

import math
from dataclasses import dataclass

from sectionproperties.pre import Geometry
from shapely import Point, Polygon

from blueprints.structural_sections._cross_section import CrossSection
from blueprints.type_alias import MM, MM2, MM3, MM4


@dataclass(frozen=True)
class CircularCrossSection(CrossSection):
    """
    Class to represent a circular cross-section using shapely for geometric calculations.

    Parameters
    ----------
    diameter : MM
        The diameter of the circular cross-section [mm].
    x : MM
        The x-coordinate of the circle's center.
    y : MM
        The y-coordinate of the circle's center.
    name : str
        The name of the rectangular cross-section, default is "Circle".
    """

    diameter: MM
    x: MM
    y: MM
    name: str = "Circle"

    def __post_init__(self) -> None:
        """Post-initialization to validate the diameter."""
        if self.diameter <= 0:
            msg = f"Diameter must be a positive value, but got {self.diameter}"
            raise ValueError(msg)

    @property
    def radius(self) -> MM:
        """
        Calculate the radius of the circular cross-section [mm].

        Returns
        -------
        MM
            The radius of the circle.
        """
        return self.diameter / 2.0

    @property
    def polygon(self) -> Polygon:
        """
        Shapely Polygon representing the circular cross-section.

        Returns
        -------
        Polygon
            The shapely Polygon representing the circle.
        """
        return self.centroid.buffer(self.radius)

    @property
    def area(self) -> MM2:
        """
        Calculate the area of the circular cross-section [mm²].

        Returns
        -------
        MM2
            The area of the circle.
        """
        return math.pi * self.radius**2.0

    @property
    def perimeter(self) -> MM:
        """
        Calculate the perimeter (circumference) of the circular cross-section [mm].

        Returns
        -------
        MM
            The perimeter of the circle.
        """
        return math.pi * self.radius * 2.0

    @property
    def centroid(self) -> Point:
        """
        Get the centroid of the circular cross-section.

        Returns
        -------
        Point
            The centroid of the circle.
        """
        return Point(self.x, self.y)

    @property
    def moment_of_inertia_about_y(self) -> MM4:
        """
        Moments of inertia of the cross-section about the y-axis [mm⁴].

        Returns
        -------
        MM4
            The moment of inertia about the y-axis.
        """
        return (math.pi / 64) * self.diameter**4

    @property
    def moment_of_inertia_about_z(self) -> MM4:
        """
        Moments of inertia of the cross-section about the z-axis [mm⁴].

        Returns
        -------
        MM4
            The moment of inertia about the z-axis.
        """
        return (math.pi / 64) * self.diameter**4

    @property
    def elastic_section_modulus_about_y_positive(self) -> MM3:
        """
        Elastic section modulus about the y-axis on the positive z side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the y-axis.
        """
        return self.moment_of_inertia_about_y / (self.diameter / 2)

    @property
    def elastic_section_modulus_about_y_negative(self) -> MM3:
        """
        Elastic section modulus about the y-axis on the negative z side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the y-axis.
        """
        return self.moment_of_inertia_about_y / (self.diameter / 2)

    @property
    def elastic_section_modulus_about_z_positive(self) -> MM3:
        """
        Elastic section modulus about the z-axis on the positive y side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the z-axis.
        """
        return self.moment_of_inertia_about_z / (self.diameter / 2)

    @property
    def elastic_section_modulus_about_z_negative(self) -> MM3:
        """
        Elastic section modulus about the z-axis on the negative y side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the z-axis.
        """
        return self.moment_of_inertia_about_z / (self.diameter / 2)

    @property
    def plastic_section_modulus_about_y(self) -> MM3:
        """
        Plastic section modulus about the y-axis [mm³].

        Returns
        -------
        MM3
            The plastic section modulus about the y-axis.
        """
        return (self.diameter**3) / 6

    @property
    def plastic_section_modulus_about_z(self) -> MM3:
        """
        Plastic section modulus about the z-axis [mm³].

        Returns
        -------
        MM3
            The plastic section modulus about the z-axis.
        """
        return (self.diameter**3) / 6

    def geometry(
        self,
        mesh_size: MM | None = None,
    ) -> Geometry:
        """Return the geometry of the circular cross-section.

        Properties
        ----------
        mesh_size : MM
            Maximum mesh element area to be used within
            the Geometry-object finite-element mesh. If not provided, a default value will be used.

        """
        if mesh_size is None:
            minimum_mesh_size = 2.0
            mesh_length = max(self.diameter / 20, minimum_mesh_size)
            mesh_size = mesh_length**2

        circular = Geometry(geom=self.polygon)
        circular.create_mesh(mesh_sizes=mesh_size)
        return circular



"""Test suite for the CHSSteelProfile class."""

import pytest
from matplotlib import pyplot as plt
from matplotlib.figure import Figure

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.steel.steel_cross_sections.chs_profile import CHSSteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.chs import CHS


class TestCHSSteelProfile:
    """Test suite for CHSSteelProfile."""

    def test_name(self, chs_profile: CHSSteelProfile) -> None:
        """Test the name of the CHS profile."""
        expected_name: str = "CHS 508x16"
        assert chs_profile.name == expected_name

    def test_code(self, chs_profile: CHSSteelProfile) -> None:
        """Test the code of the CHS profile."""
        expected_alias: str = "CHS 508x16"
        assert chs_profile.name == expected_alias

    def test_steel_volume_per_meter(self, chs_profile: CHSSteelProfile) -> None:
        """Test the steel volume per meter."""
        expected_volume: float = 2.47e-2  # m³/m
        assert pytest.approx(chs_profile.volume_per_meter, rel=1e-2) == expected_volume

    def test_steel_weight_per_meter(self, chs_profile: CHSSteelProfile) -> None:
        """Test the steel weight per meter."""
        expected_weight: float = 2.47e-2 * 7850  # kg/m
        assert pytest.approx(chs_profile.weight_per_meter, rel=1e-2) == expected_weight

    def test_area(self, chs_profile: CHSSteelProfile) -> None:
        """Test the steel cross-sectional area."""
        expected_area: float = 2.47e4  # mm²
        assert pytest.approx(chs_profile.area, rel=1e-2) == expected_area

    def test_centroid(self, chs_profile: CHSSteelProfile) -> None:
        """Test the centroid of the steel cross-section."""
        expected_centroid: tuple[float, float] = (0, 0)  # (x, y) coordinates
        assert pytest.approx(chs_profile.centroid.x, rel=1e-2) == expected_centroid[0]
        assert pytest.approx(chs_profile.centroid.y, rel=1e-2) == expected_centroid[1]

    def test_moment_of_inertia_about_y(self, chs_profile: CHSSteelProfile) -> None:
        """Test the moment of inertia about the y-axis."""
        expected_moi_y: float = 7.4909e8  # mm⁴
        assert pytest.approx(chs_profile.moment_of_inertia_about_y, rel=1e-2) == expected_moi_y

    def test_moment_of_inertia_about_z(self, chs_profile: CHSSteelProfile) -> None:
        """Test the moment of inertia about the z-axis."""
        expected_moi_z: float = 7.4909e8  # mm⁴
        assert pytest.approx(chs_profile.moment_of_inertia_about_z, rel=1e-2) == expected_moi_z

    def test_elastic_section_modulus_about_y_positive(self, chs_profile: CHSSteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the positive z side."""
        expected_modulus_y_positive: float = 2.9490e6  # mm³
        assert pytest.approx(chs_profile.elastic_section_modulus_about_y_positive, rel=1e-2) == expected_modulus_y_positive

    def test_elastic_section_modulus_about_y_negative(self, chs_profile: CHSSteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the negative z side."""
        expected_modulus_y_negative: float = 2.9490e6  # mm³
        assert pytest.approx(chs_profile.elastic_section_modulus_about_y_negative, rel=1e-2) == expected_modulus_y_negative

    def test_elastic_section_modulus_about_z_positive(self, chs_profile: CHSSteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the positive y side."""
        expected_modulus_z_positive: float = 2.9490e6  # mm³
        assert pytest.approx(chs_profile.elastic_section_modulus_about_z_positive, rel=1e-2) == expected_modulus_z_positive

    def test_elastic_section_modulus_about_z_negative(self, chs_profile: CHSSteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the negative y side."""
        expected_modulus_z_negative: float = 2.9490e6  # mm³
        assert pytest.approx(chs_profile.elastic_section_modulus_about_z_negative, rel=1e-2) == expected_modulus_z_negative

    def test_plot(self, chs_profile: CHSSteelProfile) -> None:
        """Test the plot method (ensure it runs without errors)."""
        fig: Figure = chs_profile.plot()
        assert isinstance(fig, plt.Figure)

    def test_geometry(self, chs_profile: CHSSteelProfile) -> None:
        """Test the geometry of the CHS profile."""
        expected_geometry = chs_profile.geometry
        assert expected_geometry is not None

    def test_section_properties(self, chs_profile: CHSSteelProfile) -> None:
        """Test the section properties of the CHS profile."""
        section_properties = chs_profile.section_properties()
        assert section_properties.mass == pytest.approx(expected=chs_profile.area, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=chs_profile.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=chs_profile.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=chs_profile.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=chs_profile.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=chs_profile.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=chs_profile.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=chs_profile.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=chs_profile.elastic_section_modulus_about_z_negative, rel=1e-2)

    def test_get_profile_with_corrosion(self) -> None:
        """Test the CHS profile with 20 mm corrosion applied."""
        # Ensure the profile raises an error if fully corroded
        with pytest.raises(ValueError, match="The profile has fully corroded."):
            CHSSteelProfile.from_standard_profile(
                profile=CHS.CHS508x16,
                steel_material=SteelMaterial(SteelStrengthClass.S355),
                corrosion_outside=5,  # mm
                corrosion_inside=11,  # mm
            )

    def test_corrosion_in_name(self) -> None:
        """Test that the name includes corrosion information."""
        chs_profile_with_corrosion = CHSSteelProfile.from_standard_profile(
            profile=CHS.CHS508x16,
            steel_material=SteelMaterial(SteelStrengthClass.S355),
            corrosion_outside=1,  # mm
            corrosion_inside=2,  # mm
        )
        expected_name_with_corrosion = "CHS 508x16 (corrosion  in: 2 mm, out: 1 mm)"
        assert chs_profile_with_corrosion.name == expected_name_with_corrosion



"""Square minus quarter circle shape: Quarter Circular Spandrel."""

import math
from dataclasses import dataclass

import numpy as np
from sectionproperties.pre import Geometry
from shapely.geometry import Point, Polygon

from blueprints.structural_sections._cross_section import CrossSection
from blueprints.type_alias import MM, MM2, MM3, MM4


@dataclass(frozen=True)
class QuarterCircularSpandrelCrossSection(CrossSection):
    """
    Class to represent a square cross-section with a quarter circle cutout for geometric calculations, named as Quarter Circular Spandrel .

    Parameters
    ----------
    radius : MM
        The length of the two sides of the cross-section.
    x : MM
        The x-coordinate of the 90-degree angle. Default is 0.
    y : MM
        The y-coordinate of the 90-degree angle. Default is 0.
    mirrored_horizontally : bool
        Whether the shape is mirrored horizontally. Default is False.
    mirrored_vertically : bool
        Whether the shape is mirrored vertically. Default is False.
    name : str
        The name of the radius cross-section. Default is "QCS".
    """

    radius: MM
    x: MM = 0
    y: MM = 0
    mirrored_horizontally: bool = False
    mirrored_vertically: bool = False
    name: str = "QCS"

    @property
    def polygon(self) -> Polygon:
        """
        Shapely Polygon representing the cross-section.

        Returns
        -------
        Polygon
            The shapely Polygon representing the shape.
        """
        left_lower = (self.x, self.y)

        # Approximate the quarter circle with 25 straight lines.
        # This resolution was chosen to balance performance and accuracy.
        # Increasing the number of segments (e.g., to 50) would improve accuracy but at the cost of computational performance.
        # Ensure this resolution meets the requirements of your specific application before using.
        quarter_circle_points = [
            (self.x + self.radius - self.radius * math.cos(math.pi / 2 * i / 25), self.y + self.radius - self.radius * math.sin(math.pi / 2 * i / 25))
            for i in range(26)
        ]
        for i in range(26):
            if self.mirrored_horizontally:
                quarter_circle_points[i] = (2 * left_lower[0] - quarter_circle_points[i][0], quarter_circle_points[i][1])
            if self.mirrored_vertically:
                quarter_circle_points[i] = (quarter_circle_points[i][0], 2 * left_lower[1] - quarter_circle_points[i][1])

        return Polygon([left_lower, *quarter_circle_points])

    @property
    def area(self) -> MM2:
        """
        Calculate the area of the cross-section.

        Returns
        -------
        MM2
            The area of the shape.
        """
        return self.radius**2 - (math.pi * self.radius**2 / 4)

    @property
    def perimeter(self) -> MM:
        """
        Calculate the perimeter of the cross-section.

        Returns
        -------
        MM
            The perimeter of the shape.
        """
        return 2 * self.radius + (math.pi * self.radius / 2)

    @property
    def centroid(self) -> Point:
        """
        Get the centroid of the cross-section, taking into account the mirrored status.

        Returns
        -------
        Point
            The centroid of the shape.
        """
        if self.radius == 0:
            return Point(self.x, self.y)

        centroid_x = (10 - 3 * np.pi) / (12 - 3 * np.pi) * self.radius + self.x
        centroid_y = (10 - 3 * np.pi) / (12 - 3 * np.pi) * self.radius + self.y

        if self.mirrored_horizontally:
            centroid_x = 2 * self.x - centroid_x
        if self.mirrored_vertically:
            centroid_y = 2 * self.y - centroid_y

        return Point(centroid_x, centroid_y)

    @property
    def moment_of_inertia_about_y(self) -> MM4:
        """
        Moments of inertia of the cross-section about the y-axis [mm⁴].

        Returns
        -------
        MM4
            The moment of inertia about the y-axis.
        """
        return (9 * np.pi**2 - 84 * np.pi + 176) / (144 * (4 - np.pi)) * self.radius**4

    @property
    def moment_of_inertia_about_z(self) -> MM4:
        """
        Moments of inertia of the cross-section about the z-axis [mm⁴].

        Returns
        -------
        MM4
            The moment of inertia about the z-axis.
        """
        return (9 * np.pi**2 - 84 * np.pi + 176) / (144 * (4 - np.pi)) * self.radius**4

    @property
    def elastic_section_modulus_about_y_positive(self) -> MM3:
        """
        Elastic section modulus about the y-axis on the positive z side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the y-axis.
        """
        distance_to_end = max(y for _, y in self.polygon.exterior.coords) - self.centroid.y
        return self.moment_of_inertia_about_y / distance_to_end if self.area != 0 else 0

    @property
    def elastic_section_modulus_about_y_negative(self) -> MM3:
        """
        Elastic section modulus about the y-axis on the negative z side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the y-axis.
        """
        distance_to_end = self.centroid.y - min(y for _, y in self.polygon.exterior.coords)
        return self.moment_of_inertia_about_y / distance_to_end if self.area != 0 else 0

    @property
    def elastic_section_modulus_about_z_positive(self) -> MM3:
        """
        Elastic section modulus about the z-axis on the positive y side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the z-axis.
        """
        distance_to_end = max(x for x, _ in self.polygon.exterior.coords) - self.centroid.x
        return self.moment_of_inertia_about_z / distance_to_end if self.area != 0 else 0

    @property
    def elastic_section_modulus_about_z_negative(self) -> MM3:
        """
        Elastic section modulus about the z-axis on the negative y side [mm³].

        Returns
        -------
        MM3
            The elastic section modulus about the z-axis.
        """
        distance_to_end = self.centroid.x - min(x for x, _ in self.polygon.exterior.coords)
        return self.moment_of_inertia_about_z / distance_to_end if self.area != 0 else 0

    @property
    def plastic_section_modulus_about_y(self) -> MM3:
        """
        Plastic section modulus about the y-axis [mm³].
        Note: This is an approximation based on a very small mesh.

        Returns
        -------
        MM3
            The plastic section modulus about the y-axis.
        """
        return self.radius**3 / 31.6851045070407

    @property
    def plastic_section_modulus_about_z(self) -> MM3:
        """
        Plastic section modulus about the z-axis [mm³].
        Note: This is an approximation based on a very small mesh.

        Returns
        -------
        MM3
            The plastic section modulus about the z-axis.
        """
        return self.radius**3 / 31.6851045070407

    def geometry(
        self,
        mesh_size: MM | None = None,
    ) -> Geometry:
        """Return the geometry of the square-with-cutout cross-section.

        Properties
        ----------
        mesh_size : MM
            Maximum mesh element area to be used within
            the Geometry-object finite-element mesh. If not provided, a default value will be used.

        """
        if mesh_size is None:
            minimum_mesh_size = 1.0
            mesh_length = max(self.radius / 5, minimum_mesh_size)
            mesh_size = mesh_length**2

        square_with_cutout = Geometry(geom=self.polygon)
        square_with_cutout.create_mesh(mesh_sizes=mesh_size)
        return square_with_cutout



"""Test suite for ISteelProfile."""

import pytest
from matplotlib import pyplot as plt
from matplotlib.figure import Figure

from blueprints.codes.eurocode.en_1993_1_1_2005.chapter_3_materials.table_3_1 import SteelStrengthClass
from blueprints.materials.steel import SteelMaterial
from blueprints.structural_sections.steel.steel_cross_sections.i_profile import ISteelProfile
from blueprints.structural_sections.steel.steel_cross_sections.standard_profiles.heb import HEB


class TestISteelProfile:
    """Test suite for ISteelProfile."""

    def test_alias(self, i_profile: ISteelProfile) -> None:
        """Test the alias of the I-profile."""
        expected_alias = "HEB360"
        assert i_profile.name == expected_alias

    def test_steel_volume_per_meter(self, i_profile: ISteelProfile) -> None:
        """Test the steel volume per meter."""
        expected_volume = 1.806e-2  # m³/m
        assert pytest.approx(i_profile.volume_per_meter, rel=1e-2) == expected_volume

    def test_steel_weight_per_meter(self, i_profile: ISteelProfile) -> None:
        """Test the steel weight per meter."""
        expected_weight = 1.806e-2 * 7850  # kg/m
        assert pytest.approx(i_profile.weight_per_meter, rel=1e-2) == expected_weight

    def test_steel_area(self, i_profile: ISteelProfile) -> None:
        """Test the steel cross-sectional area."""
        expected_area = 1.806e4  # mm²
        assert pytest.approx(i_profile.area, rel=1e-2) == expected_area

    def test_centroid(self, i_profile: ISteelProfile) -> None:
        """Test the centroid of the steel cross-section."""
        expected_centroid = (0, 0)  # (x, y) coordinates
        assert pytest.approx(i_profile.centroid.x, rel=1e-2) == expected_centroid[0]
        assert pytest.approx(i_profile.centroid.y, rel=1e-2) == expected_centroid[1]

    def test_moment_of_inertia_about_y(self, i_profile: ISteelProfile) -> None:
        """Test the moment of inertia about the y-axis."""
        expected_moi_y = 4.319e8  # mm⁴
        assert pytest.approx(i_profile.moment_of_inertia_about_y, rel=1e-2) == expected_moi_y

    def test_moment_of_inertia_about_z(self, i_profile: ISteelProfile) -> None:
        """Test the moment of inertia about the z-axis."""
        expected_moi_z = 1.014e8  # mm⁴
        assert pytest.approx(i_profile.moment_of_inertia_about_z, rel=1e-2) == expected_moi_z

    def test_elastic_section_modulus_about_y_positive(self, i_profile: ISteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the positive z side."""
        expected_modulus_y_positive = 2.4e6  # mm³
        assert pytest.approx(i_profile.elastic_section_modulus_about_y_positive, rel=1e-2) == expected_modulus_y_positive

    def test_elastic_section_modulus_about_y_negative(self, i_profile: ISteelProfile) -> None:
        """Test the elastic section modulus about the y-axis on the negative z side."""
        expected_modulus_y_negative = 2.4e6  # mm³
        assert pytest.approx(i_profile.elastic_section_modulus_about_y_negative, rel=1e-2) == expected_modulus_y_negative

    def test_elastic_section_modulus_about_z_positive(self, i_profile: ISteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the positive y side."""
        expected_modulus_z_positive = 6.761e5  # mm³
        assert pytest.approx(i_profile.elastic_section_modulus_about_z_positive, rel=1e-2) == expected_modulus_z_positive

    def test_elastic_section_modulus_about_z_negative(self, i_profile: ISteelProfile) -> None:
        """Test the elastic section modulus about the z-axis on the negative y side."""
        expected_modulus_z_negative = 6.761e5  # mm³
        assert pytest.approx(i_profile.elastic_section_modulus_about_z_negative, rel=1e-2) == expected_modulus_z_negative

    def test_plot(self, i_profile: ISteelProfile) -> None:
        """Test the plot method (ensure it runs without errors)."""
        fig: Figure = i_profile.plot()
        assert isinstance(fig, plt.Figure)

    def test_geometry(self, i_profile: ISteelProfile) -> None:
        """Test the geometry of the I profile."""
        expected_geometry = i_profile.geometry
        assert expected_geometry is not None

    def test_section_properties(self, i_profile: ISteelProfile) -> None:
        """Test the section properties of the I profile."""
        section_properties = i_profile.section_properties()
        assert section_properties.mass == pytest.approx(expected=i_profile.area, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=i_profile.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=i_profile.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=i_profile.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=i_profile.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=i_profile.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=i_profile.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=i_profile.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=i_profile.elastic_section_modulus_about_z_negative, rel=1e-2)

    def test_get_profile_with_corrosion(self) -> None:
        """Test the EHB profile with 20 mm corrosion applied."""
        # Ensure the profile raises an error if fully corroded
        with pytest.raises(ValueError, match="The profile has fully corroded."):
            ISteelProfile.from_standard_profile(
                profile=HEB.HEB360,
                steel_material=SteelMaterial(SteelStrengthClass.S355),
                corrosion=20,  # mm
            )

    def test_corrosion_in_name(self) -> None:
        """Test that the name includes corrosion information."""
        i_profile_with_corrosion = ISteelProfile.from_standard_profile(
            profile=HEB.HEB360,
            steel_material=SteelMaterial(SteelStrengthClass.S355),
            corrosion=2,  # mm
        )
        expected_name_with_corrosion = "HEB360 (corrosion: 2 mm)"
        assert i_profile_with_corrosion.name == expected_name_with_corrosion



"""Tests for constants classes for the calculation of nominal concrete cover."""

from dataclasses import dataclass, field

import pytest

from blueprints.checks.nominal_concrete_cover.constants.base import NominalConcreteCoverConstantsBase
from blueprints.checks.nominal_concrete_cover.constants.constants_en_1992_1_1_2004 import NominalConcreteCoverConstants
from blueprints.checks.nominal_concrete_cover.constants.constants_nen_en_1992_1_1_a1_2020 import NominalConcreteCoverConstants2020A1
from blueprints.checks.nominal_concrete_cover.definitions import AbrasionClass, CastingSurface
from blueprints.type_alias import MM


@dataclass(frozen=True)
class DummyConstants(NominalConcreteCoverConstantsBase):
    """Dummy constants for testing purposes."""

    CODE_SUFFIX: str = "DUMMY"
    COVER_INCREASE_FOR_ABRASION_CLASS: dict[AbrasionClass, MM] = field(default_factory=dict)
    COVER_INCREASE_FOR_UNEVEN_SURFACE: MM = 5
    DEFAULT_DELTA_C_DEV: MM = 5

    def __post_init__(self) -> None:
        """Post-initialization method to set default values."""
        if not self.COVER_INCREASE_FOR_ABRASION_CLASS:
            abrasion_class_cover_increase = {
                "NA": 0,
                "XM1": 5,
                "XM2": 10,
                "XM3": 15,
            }
            object.__setattr__(self, "COVER_INCREASE_FOR_ABRASION_CLASS", abrasion_class_cover_increase)

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface(c_min_dur, casting_surface) -> MM:  # noqa: ANN001, ARG004
        """Dummy method for testing purposes."""
        return 0

    @staticmethod
    def minimum_cover_with_regard_to_casting_surface_latex(casting_surface) -> str:  # noqa: ANN001, ARG004
        """Dummy method for testing purposes."""
        return ""


class TestNominalConcreteCoverConstantsBase:
    """Tests for the base class for constants for the calculation of nominal concrete cover."""

    def test_abstract_method(self) -> None:
        """Test that the abstract method raises a TypeError."""
        with pytest.raises(TypeError):
            _ = NominalConcreteCoverConstantsBase("_", 0, {}, 0)  # type: ignore[abstract]

    def test_instantiation(self) -> None:
        """Test that the class can be instantiated."""
        constants = DummyConstants()
        assert constants.COVER_INCREASE_FOR_UNEVEN_SURFACE == 5
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            "NA": 0,
            "XM1": 5,
            "XM2": 10,
            "XM3": 15,
        }
        assert constants.DEFAULT_DELTA_C_DEV == 5
        assert constants.CODE_SUFFIX == "DUMMY"
        assert constants.minimum_cover_with_regard_to_casting_surface(0, CastingSurface.PERMANENTLY_EXPOSED) == 0
        assert constants.minimum_cover_with_regard_to_casting_surface_latex(CastingSurface.PERMANENTLY_EXPOSED) == ""


class TestNominalConcreteCoverConstants2004:
    """Tests for the constants class for the calculation of nominal concrete cover according to EN 1992-1-1:2004."""

    def test_instantiation(self) -> None:
        """Test that the class can be instantiated."""
        constants = NominalConcreteCoverConstants()
        assert constants.COVER_INCREASE_FOR_UNEVEN_SURFACE == 5
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 5,
            AbrasionClass.XM2: 10,
            AbrasionClass.XM3: 15,
        }
        assert constants.DEFAULT_DELTA_C_DEV == 10

    def test_instantiation_with_custom_cover_increase_uneven_surfaces(self) -> None:
        """Test that the class cannot be instantiated with custom value for the cover increase for uneven surfaces."""
        with pytest.raises(TypeError, match=".* got an unexpected keyword argument .*"):
            _ = NominalConcreteCoverConstants(  # type: ignore[call-arg]
                COVER_INCREASE_FOR_UNEVEN_SURFACE=10,
            )

    def test_instantiation_with_custom_cover_increase_abraison_class(self) -> None:
        """Test that the class can be instantiated with custom value for the cover increase for abrasion class."""
        custom_constants = NominalConcreteCoverConstants(  # type: ignore[call-arg]
            COVER_INCREASE_FOR_ABRASION_CLASS={
                AbrasionClass.NA: 0,
            },
        )
        assert custom_constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
        }

    def test_instantiation_with_custom_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with custom value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants(  # type: ignore[call-arg]
            DEFAULT_DELTA_C_DEV=13,
        )
        assert custom_constants.DEFAULT_DELTA_C_DEV == 13

    def test_instantiation_with_default_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with default value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants()
        assert custom_constants.DEFAULT_DELTA_C_DEV == 10  # Default value

    def test_post_init(self) -> None:
        """Test that the post-init method sets default values."""
        constants = NominalConcreteCoverConstants()
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 5,
            AbrasionClass.XM2: 10,
            AbrasionClass.XM3: 15,
        }

    @pytest.mark.parametrize(
        ("c_min_dur", "casting_surface", "expected_result"),
        [
            (10, CastingSurface.PERMANENTLY_EXPOSED, 0),
            (10, CastingSurface.FORMWORK, 0),
            (10, CastingSurface.PREPARED_GROUND, 50),
            (10, CastingSurface.DIRECTLY_AGAINST_SOIL, 85),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface(self, c_min_dur: MM, casting_surface: CastingSurface, expected_result: MM) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface."""
        constants = NominalConcreteCoverConstants()
        assert constants.minimum_cover_with_regard_to_casting_surface(c_min_dur, casting_surface) == expected_result

    @pytest.mark.parametrize(
        ("casting_surface", "expected_result"),
        [
            (CastingSurface.PERMANENTLY_EXPOSED, "0 (No additional requirements for Permanently exposed)"),
            (CastingSurface.FORMWORK, "0 (No additional requirements for Formwork)"),
            (CastingSurface.PREPARED_GROUND, "k1 \\ge c_{min,dur} + 40 mm for Prepared ground (including blinding)"),
            (CastingSurface.DIRECTLY_AGAINST_SOIL, "k2 \\ge c_{min,dur} + 75 mm for Directly against soil"),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface_latex(self, casting_surface: CastingSurface, expected_result: str) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface in LaTeX."""
        constants = NominalConcreteCoverConstants()
        assert constants.minimum_cover_with_regard_to_casting_surface_latex(casting_surface) == expected_result


class TestNominalConcreteCoverConstants2020A1:
    """Tests for the constants class for the calculation of nominal concrete cover according to NEN-EN 1992-1-1:2005+A1:2015+NB:2016+A1:2020."""

    def test_instantiation(self) -> None:
        """Test that the class can be instantiated."""
        constants = NominalConcreteCoverConstants2020A1()
        assert constants.COVER_INCREASE_FOR_UNEVEN_SURFACE == 5
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 0,
            AbrasionClass.XM2: 0,
            AbrasionClass.XM3: 0,
        }
        assert constants.DEFAULT_DELTA_C_DEV == 5

    def test_instantiation_with_custom_cover_increase_uneven_surfaces(self) -> None:
        """Test that the class cannot be instantiated with custom value for the cover increase for uneven surfaces."""
        with pytest.raises(TypeError, match=".* got an unexpected keyword argument .*"):
            _ = NominalConcreteCoverConstants2020A1(  # type: ignore[call-arg]
                COVER_INCREASE_FOR_UNEVEN_SURFACE=10,
            )

    def test_instantiation_with_custom_cover_increase_abraison_class(self) -> None:
        """Test that the class can be instantiated with custom value for the cover increase for abrasion class."""
        custom_constants = NominalConcreteCoverConstants2020A1(  # type: ignore[call-arg]
            COVER_INCREASE_FOR_ABRASION_CLASS={
                AbrasionClass.NA: 0,
            },
        )
        assert custom_constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
        }

    def test_instantiation_with_custom_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with custom value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants2020A1(  # type: ignore[call-arg]
            DEFAULT_DELTA_C_DEV=13,
        )
        assert custom_constants.DEFAULT_DELTA_C_DEV == 13

    def test_instantiation_with_default_delta_c_dev(self) -> None:
        """Test that the class can be instantiated with default value for the default delta c_dev."""
        custom_constants = NominalConcreteCoverConstants2020A1()
        assert custom_constants.DEFAULT_DELTA_C_DEV == 5  # Default value

    def test_post_init(self) -> None:
        """Test that the post-init method sets default values."""
        constants = NominalConcreteCoverConstants2020A1()
        assert constants.COVER_INCREASE_FOR_ABRASION_CLASS == {
            AbrasionClass.NA: 0,
            AbrasionClass.XM1: 0,
            AbrasionClass.XM2: 0,
            AbrasionClass.XM3: 0,
        }

    @pytest.mark.parametrize(
        ("c_min_dur", "casting_surface", "expected_result"),
        [
            (10, CastingSurface.PERMANENTLY_EXPOSED, 0),
            (10, CastingSurface.FORMWORK, 0),
            (10, CastingSurface.PREPARED_GROUND, 20),
            (10, CastingSurface.DIRECTLY_AGAINST_SOIL, 60),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface(self, c_min_dur: MM, casting_surface: CastingSurface, expected_result: MM) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface."""
        constants = NominalConcreteCoverConstants2020A1()
        assert constants.minimum_cover_with_regard_to_casting_surface(c_min_dur, casting_surface) == expected_result

    @pytest.mark.parametrize(
        ("casting_surface", "expected_result"),
        [
            (CastingSurface.PERMANENTLY_EXPOSED, "0 (No additional requirements for Permanently exposed)"),
            (CastingSurface.FORMWORK, "0 (No additional requirements for Formwork)"),
            (CastingSurface.PREPARED_GROUND, "k1 \\ge c_{min,dur} + 10 mm for Prepared ground (including blinding)"),
            (CastingSurface.DIRECTLY_AGAINST_SOIL, "k2 \\ge c_{min,dur} + 50 mm for Directly against soil"),
        ],
    )
    def test_minimum_cover_with_regard_to_casting_surface_latex(self, casting_surface: CastingSurface, expected_result: str) -> None:
        """Test the method for the calculation of the minimum cover with regard to casting surface in LaTeX."""
        constants = NominalConcreteCoverConstants2020A1()
        assert constants.minimum_cover_with_regard_to_casting_surface_latex(casting_surface) == expected_result



"""Tests for right-angled triangular cross-section shapes with a quarter circle."""

import numpy as np
import pytest
from shapely.geometry import Point

from blueprints.structural_sections.cross_section_quarter_circular_spandrel import QuarterCircularSpandrelCrossSection


class TestQuarterCircularSpandrelCrossSection:
    """Tests for the QuarterCircularSpandrelCrossSection class."""

    def test_area(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the area property of the QuarterCircularSpandrelCrossSection class."""
        expected_area = 50.0**2 - (np.pi * 50.0**2 / 4)
        assert qcs_cross_section.area == pytest.approx(expected=expected_area, rel=1e-6)

    def test_perimeter(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the perimeter property of the QuarterCircularSpandrelCrossSection class."""
        expected_perimeter = 2 * 50.0 + (np.pi * 50.0 / 2)
        assert qcs_cross_section.perimeter == pytest.approx(expected=expected_perimeter, rel=1e-6)

    def test_centroid(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the centroid property of the QuarterCircularSpandrelCrossSection class."""
        expected_centroid = Point(111.1683969472876, 261.1683969472876)
        assert qcs_cross_section.centroid == pytest.approx(expected=expected_centroid, rel=1e-6)

    def test_moments_of_inertia(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the moments of inertia properties of the QuarterCircularSpandrelCrossSection class."""
        expected_inertia = (9 * np.pi**2 - 84 * np.pi + 176) / (144 * (4 - np.pi)) * 50**4

        assert qcs_cross_section.moment_of_inertia_about_y == pytest.approx(expected=expected_inertia, rel=1e-6)
        assert qcs_cross_section.moment_of_inertia_about_z == pytest.approx(expected=expected_inertia, rel=1e-6)

    def test_section_moduli(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the section moduli properties of the QuarterCircularSpandrelCrossSection class."""
        expected_modulus_positive = qcs_cross_section.moment_of_inertia_about_y / (
            qcs_cross_section.radius - qcs_cross_section.centroid.y + qcs_cross_section.y
        )
        expected_modulus_negative = qcs_cross_section.moment_of_inertia_about_y / (qcs_cross_section.centroid.y - qcs_cross_section.y)
        assert qcs_cross_section.elastic_section_modulus_about_y_positive == pytest.approx(expected=expected_modulus_positive, rel=1e-6)
        assert qcs_cross_section.elastic_section_modulus_about_y_negative == pytest.approx(expected=expected_modulus_negative, rel=1e-6)
        assert qcs_cross_section.elastic_section_modulus_about_z_positive == pytest.approx(expected=expected_modulus_positive, rel=1e-6)
        assert qcs_cross_section.elastic_section_modulus_about_z_negative == pytest.approx(expected=expected_modulus_negative, rel=1e-6)

    def test_plastic_section_moduli(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the plastic section moduli properties of the QuarterCircularSpandrelCrossSection class."""
        expected_plastic_modulus = (50.0**3) / 31.6851045070407
        assert qcs_cross_section.plastic_section_modulus_about_y == pytest.approx(expected=expected_plastic_modulus, rel=1e-6)
        assert qcs_cross_section.plastic_section_modulus_about_z == pytest.approx(expected=expected_plastic_modulus, rel=1e-6)

    def test_polygon(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the geometry property of the QuarterCircularSpandrelCrossSection class."""
        polygon = qcs_cross_section.polygon
        assert polygon.is_valid
        assert polygon.area == pytest.approx(expected=qcs_cross_section.area, rel=1e-2)

    def test_mirrored_geometry(self) -> None:
        """Test the geometry property for flipped cross-sections."""
        # Test flipped horizontally
        qcs_cross_section = QuarterCircularSpandrelCrossSection(radius=50.0, x=0.0, y=0.0)
        qcs_cross_section_mirrored_horizontally = QuarterCircularSpandrelCrossSection(radius=50.0, x=0.0, y=0.0, mirrored_horizontally=True)
        assert qcs_cross_section_mirrored_horizontally.polygon.centroid.x == pytest.approx(expected=-qcs_cross_section.polygon.centroid.x, rel=1e-3)
        assert qcs_cross_section_mirrored_horizontally.area == pytest.approx(expected=qcs_cross_section.area, rel=1e-3)
        assert qcs_cross_section_mirrored_horizontally.centroid.x == pytest.approx(expected=-qcs_cross_section.centroid.x, rel=1e-3)
        assert qcs_cross_section_mirrored_horizontally.centroid.y == pytest.approx(expected=qcs_cross_section.centroid.y, rel=1e-3)

        # Test flipped vertically
        qcs_cross_section_mirrored_vertically = QuarterCircularSpandrelCrossSection(radius=50.0, x=0.0, y=0.0, mirrored_vertically=True)
        assert qcs_cross_section_mirrored_vertically.polygon.centroid.y == pytest.approx(expected=-qcs_cross_section.polygon.centroid.y, rel=1e-3)
        assert qcs_cross_section_mirrored_vertically.area == pytest.approx(expected=qcs_cross_section.area, rel=1e-3)
        assert qcs_cross_section_mirrored_vertically.centroid.x == pytest.approx(expected=qcs_cross_section.centroid.x, rel=1e-3)
        assert qcs_cross_section_mirrored_vertically.centroid.y == pytest.approx(expected=-qcs_cross_section.centroid.y, rel=1e-3)

        # Test flipped both horizontally and vertically
        qcs_cross_section_mirrored_both = QuarterCircularSpandrelCrossSection(
            radius=50.0, x=0.0, y=0.0, mirrored_horizontally=True, mirrored_vertically=True
        )
        assert qcs_cross_section_mirrored_both.polygon.centroid.x == pytest.approx(expected=-qcs_cross_section.polygon.centroid.x, rel=1e-3)
        assert qcs_cross_section_mirrored_both.polygon.centroid.y == pytest.approx(expected=-qcs_cross_section.polygon.centroid.y, rel=1e-3)
        assert qcs_cross_section_mirrored_both.area == pytest.approx(expected=qcs_cross_section.area, rel=1e-3)
        assert qcs_cross_section_mirrored_both.centroid.x == pytest.approx(expected=-qcs_cross_section.centroid.x, rel=1e-3)
        assert qcs_cross_section_mirrored_both.centroid.y == pytest.approx(expected=-qcs_cross_section.centroid.y, rel=1e-3)

    def test_radius_is_zero(self) -> None:
        """Test the behavior when radius is zero."""
        qcs_cross_section_zero_radius = QuarterCircularSpandrelCrossSection(radius=0.0, x=0.0, y=0.0)
        assert qcs_cross_section_zero_radius.area == 0.0
        assert qcs_cross_section_zero_radius.perimeter == 0.0
        assert qcs_cross_section_zero_radius.moment_of_inertia_about_y == 0.0
        assert qcs_cross_section_zero_radius.moment_of_inertia_about_z == 0.0
        assert qcs_cross_section_zero_radius.elastic_section_modulus_about_y_positive == 0.0

    def test_section_properties(self, qcs_cross_section: QuarterCircularSpandrelCrossSection) -> None:
        """Test the section properties of the RectangularCrossSection class."""
        section_properties = qcs_cross_section.section_properties()
        assert section_properties.area == pytest.approx(expected=qcs_cross_section.area, rel=1e-2)
        assert section_properties.perimeter == pytest.approx(expected=qcs_cross_section.perimeter, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=qcs_cross_section.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=qcs_cross_section.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=qcs_cross_section.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=qcs_cross_section.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=qcs_cross_section.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=qcs_cross_section.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=qcs_cross_section.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=qcs_cross_section.elastic_section_modulus_about_z_negative, rel=1e-2)
        assert section_properties.sxx == pytest.approx(expected=qcs_cross_section.plastic_section_modulus_about_y, rel=1e-2)
        assert section_properties.syy == pytest.approx(expected=qcs_cross_section.plastic_section_modulus_about_z, rel=1e-2)



"""Tests for cross-section shapes."""

import pytest
from shapely.geometry import Polygon

from blueprints.structural_sections.cross_section_triangle import RightAngledTriangularCrossSection


class TestRightAngledTriangularCrossSection:
    """Tests for the RightAngledTriangularCrossSection class."""

    def test_area(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the area property of the RightAngledTriangularCrossSection class."""
        assert triangular_cross_section.area == pytest.approx(expected=10000.0, rel=1e-6)

    def test_polygon(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the polygon property of the RightAngledTriangularCrossSection class."""
        assert isinstance(triangular_cross_section.polygon, Polygon)

    def test_geometry(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the geometry property of the RightAngledTriangularCrossSection class."""
        geometry = triangular_cross_section.geometry()
        assert geometry is not None

    def test_perimeter(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the perimeter property of the RightAngledTriangularCrossSection class."""
        expected_perimeter = 100.0 + 200.0 + (100.0**2 + 200.0**2) ** 0.5
        assert triangular_cross_section.perimeter == pytest.approx(expected=expected_perimeter, rel=1e-6)

    def test_centroid(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the centroid property of the RightAngledTriangularCrossSection class."""
        centroid = triangular_cross_section.centroid
        assert (centroid.x, centroid.y) == pytest.approx(expected=(100.0 / 3 + 100, 200.0 / 3 + 250), rel=1e-6)

    def test_moments_of_inertia(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the moments of inertia properties of the RightAngledTriangularCrossSection class."""
        expected_y = (100.0 * 200.0**3) / 36
        expected_z = (200.0 * 100.0**3) / 36
        assert triangular_cross_section.moment_of_inertia_about_y == pytest.approx(expected=expected_y, rel=1e-6)
        assert triangular_cross_section.moment_of_inertia_about_z == pytest.approx(expected=expected_z, rel=1e-6)

    def test_section_moduli(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the section moduli properties of the RightAngledTriangularCrossSection class."""
        expected_y_positive = (100.0 * 200.0**3) / (36 * (200.0 / 3 * 2))
        expected_z_positive = (200.0 * 100.0**3) / (36 * (100.0 / 3 * 2))
        expected_y_negative = (100.0 * 200.0**3) / (36 * (200.0 / 3))
        expected_z_negative = (200.0 * 100.0**3) / (36 * (100.0 / 3))
        assert triangular_cross_section.elastic_section_modulus_about_y_positive == pytest.approx(expected=expected_y_positive, rel=1e-6)
        assert triangular_cross_section.elastic_section_modulus_about_z_positive == pytest.approx(expected=expected_z_positive, rel=1e-6)
        assert triangular_cross_section.elastic_section_modulus_about_y_negative == pytest.approx(expected=expected_y_negative, rel=1e-6)
        assert triangular_cross_section.elastic_section_modulus_about_z_negative == pytest.approx(expected=expected_z_negative, rel=1e-6)

    def test_plastic_section_moduli(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the plastic section moduli properties of the RightAngledTriangularCrossSection class."""
        assert triangular_cross_section.plastic_section_modulus_about_y == pytest.approx(expected=390529.656, rel=1e-6)
        assert triangular_cross_section.plastic_section_modulus_about_z == pytest.approx(expected=195264.828, rel=1e-6)

    def test_invalid_base(self) -> None:
        """Test initialization with an invalid base value."""
        with pytest.raises(ValueError, match="Base must be a positive value"):
            RightAngledTriangularCrossSection(name="InvalidBase", base=-10.0, height=200.0)

    def test_invalid_height(self) -> None:
        """Test initialization with an invalid height value."""
        with pytest.raises(ValueError, match="Height must be a positive value"):
            RightAngledTriangularCrossSection(name="InvalidHeight", base=100.0, height=-20.0)

    def test_mirrored_polygon(self) -> None:
        """Test the geometry property when the triangle is mirrored."""
        mirrored_triangle = RightAngledTriangularCrossSection(
            name="MirroredTriangle", base=100.0, height=200.0, mirrored_horizontally=True, mirrored_vertically=True
        )
        polygon = mirrored_triangle.polygon
        assert polygon.is_valid
        assert len(polygon.exterior.coords) == 4
        assert (polygon.exterior.coords[1][0], polygon.exterior.coords[1][1]) == (-100.0, 0)
        assert (polygon.exterior.coords[2][0], polygon.exterior.coords[2][1]) == (0, -200.0)

    def test_section_properties(self, triangular_cross_section: RightAngledTriangularCrossSection) -> None:
        """Test the section properties of the RightAngledTriangularCrossSection class."""
        section_properties = triangular_cross_section.section_properties()
        assert section_properties.area == pytest.approx(expected=triangular_cross_section.area, rel=1e-2)
        assert section_properties.perimeter == pytest.approx(expected=triangular_cross_section.perimeter, rel=1e-2)
        assert section_properties.cx == pytest.approx(expected=triangular_cross_section.centroid.x, rel=1e-2)
        assert section_properties.cy == pytest.approx(expected=triangular_cross_section.centroid.y, rel=1e-2)
        assert section_properties.ixx_c == pytest.approx(expected=triangular_cross_section.moment_of_inertia_about_y, rel=1e-2)
        assert section_properties.iyy_c == pytest.approx(expected=triangular_cross_section.moment_of_inertia_about_z, rel=1e-2)
        assert section_properties.zxx_plus == pytest.approx(expected=triangular_cross_section.elastic_section_modulus_about_y_positive, rel=1e-2)
        assert section_properties.zyy_plus == pytest.approx(expected=triangular_cross_section.elastic_section_modulus_about_z_positive, rel=1e-2)
        assert section_properties.zxx_minus == pytest.approx(expected=triangular_cross_section.elastic_section_modulus_about_y_negative, rel=1e-2)
        assert section_properties.zyy_minus == pytest.approx(expected=triangular_cross_section.elastic_section_modulus_about_z_negative, rel=1e-2)
        assert section_properties.sxx == pytest.approx(expected=triangular_cross_section.plastic_section_modulus_about_y, rel=1e-2)
        assert section_properties.syy == pytest.approx(expected=triangular_cross_section.plastic_section_modulus_about_z, rel=1e-2)



"""Formula 5.4 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN
from blueprints.validations import raise_if_negative


class Form5Dot4TransverseForceEffectBracingSystem(Formula):
    """Class representing formula 5.4 for the calculation of the effect of the inclination on bracing systems, [$H_{i}$].

    See Figure 5.1 b.
    """

    label = "5.4"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        theta_i: DIMENSIONLESS,
        n_a: KN,
        n_b: KN,
    ) -> None:
        r"""[$H_{i}$] Effect of the inclination on bracing systems [$kN$].

        EN 1992-1-1:2004 art.5.2(8) - Formula (5.4)

        Parameters
        ----------
        theta_i : DIMENSIONLESS
            [$Θ_{i}$] Eccentricity, initial inclination imperfections [-].
        n_a : KN
            [$N_{a}$] Axial force in the member [$kN$].
        n_b : KN
            [$N_{b}$] Axial force in the member [$kN$].

        Notes
        -----
        where [$N_{a}$] and [$N_{b}$] are longitudinal forces contributing to [$H_{i}$].
        Positive values for compression, tension is not allowed.
        """
        super().__init__()
        self.theta_i = theta_i
        self.n_a = n_a
        self.n_b = n_b

    @staticmethod
    def _evaluate(
        theta_i: DIMENSIONLESS,
        n_a: KN,
        n_b: KN,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            theta_i=theta_i,
            n_a=n_a,
            n_b=n_b,
        )
        return theta_i * (n_b - n_a)

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 5.4."""
        return LatexFormula(
            return_symbol=r"H_{i}",
            result=f"{self:.3f}",
            equation=r"Θ_{i} \cdot (N_{b} - N_{a})",
            numeric_equation=rf"{self.theta_i:.3f} \cdot ({self.n_b:.3f} - {self.n_a:.3f})",
            comparison_operator_label="=",
        )



"""Formula 5.6 from EN 1992-1-1:2004: Chapter 5 - Structural Analysis."""

from blueprints.codes.eurocode.en_1992_1_1_2004 import EN_1992_1_1_2004
from blueprints.codes.formula import Formula
from blueprints.codes.latex_formula import LatexFormula
from blueprints.type_alias import DIMENSIONLESS, KN
from blueprints.validations import raise_if_negative


class Form5Dot6TransverseForceEffectRoofDiaphragm(Formula):
    """Class representing formula 5.6 for the calculation of the effect of the inclination on roof diaphragm, [$H_{i}$].
    See Figure 5.1 c2.
    """

    label = "5.6"
    source_document = EN_1992_1_1_2004

    def __init__(
        self,
        theta_i: DIMENSIONLESS,
        n_a: KN,
    ) -> None:
        r"""[$H_{i}$] Effect of the inclination on roof diaphragm [$kN$].

        EN 1992-1-1:2004 art.5.2(8) - Formula (5.6)

        Parameters
        ----------
        theta_i : DIMENSIONLESS
            [$Θ_{i}$] Eccentricity, initial inclination imperfections [-].
        n_a : KN
            [$N_{a}$] Axial force in the member [$kN$].

        Notes
        -----
        where [$N_{a}$] is a longitudinal force contributing to [$H_{i}$].
        Positive values for compression, tension is not allowed.
        """
        super().__init__()
        self.theta_i = theta_i
        self.n_a = n_a

    @staticmethod
    def _evaluate(
        theta_i: DIMENSIONLESS,
        n_a: KN,
    ) -> KN:
        """Evaluates the formula, for more information see the __init__ method."""
        raise_if_negative(
            theta_i=theta_i,
            n_a=n_a,
        )
        return theta_i * n_a

    def latex(self) -> LatexFormula:
        """Returns LatexFormula object for formula 5.6."""
        return LatexFormula(
            return_symbol=r"H_{i}",
            result=f"{self:.3f}",
            equation=r"Θ_{i} \cdot N_{a}",
            numeric_equation=rf"{self.theta_i:.3f} \cdot {self.n_a:.3f}",
            comparison_operator_label="=",
        )
